﻿Imports System
Imports System.Windows
Imports System.Windows.Forms
Imports System.Drawing
Imports PanoramaPrintSample.ClassPanoramaCommon

Public Class FormPanoramaApp

    '*! -----------------------------------------------------------------------------
    '*	@section	: Constant values.
    '* ------------------------------------------------------------------------------

    ' Registry identification name
    Private Const REGKEY_STR As String = "Software\PanoramaSampleApp"

    ' Paper size. (6x8)
    Private Const PAPER_SIZE_WIDTH_6x8 As Integer = 2436
    Private Const PAPER_SIZE_HEIGHT_6x8 As Integer = 1844

    ' Paper size. (8x10)
    Private Const PAPER_SIZE_WIDTH_8x10 As Integer = 3036
    Private Const PAPER_SIZE_HEIGHT_8x10 As Integer = 2448

    ' Paper size. (8x12)
    Private Const PAPER_SIZE_WIDTH_8x12 As Integer = 3636
    Private Const PAPER_SIZE_HEIGHT_8x12 As Integer = 2448

    ' Paper size. (A4)
    Private Const PAPER_SIZE_WIDTH_A4 As Integer = 3544
    Private Const PAPER_SIZE_HEIGHT_A4 As Integer = 2560

    Private PAPER_SIZE_WIDTH As Integer = PAPER_SIZE_WIDTH_6x8
    Private PAPER_SIZE_HEIGHT As Integer = PAPER_SIZE_HEIGHT_6x8

    Private RESOLUTION_LIMIT As Single = 600.0

    Private Numeric_OverlapWidth As Single = 2.0
    Private CheckBox_CorrectEnable As Boolean = True

    '*! -----------------------------------------------------------------------------
    '*	@section	: Static variables.
    '* ------------------------------------------------------------------------------
    ' Print object.
    Private sPrintCtrl As PrinterControler

    ' Base & print panorama images.
    Private sBmpSrc As Bitmap = Nothing
    Private sBmpPanoramaTbl() As Bitmap = Nothing

    'Control flags
    Private sImgFilename As String

    'Print Cancel Flag
    Private xPrint_Cancel As Boolean = False

    Public OS_Version As Integer
    Public Const OSV_NotCompatible As Integer = 0
    Public Const OSV_Vista As Integer = 6
    Public Const OSV_Win10 As Integer = 10

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Initialize.
    '* ------------------------------------------------------------------------------
    Private Sub Initialize()

        'Initialize API
        If CheckEnvironment() = 32 Then
            PanoEnv = New ClassPanoramaModule_x86
        Else
            PanoEnv = New ClassPanoramaModule_x64
        End If

        'Set default values
        ComboBox_Resolution.SelectedIndex = 0
        ComboBox_HighDensity.SelectedIndex = 0
        ComboBox_OverCoat.SelectedIndex = 0
        sPrintCtrl = New PrinterControler

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Load  and save setting.
    '* ------------------------------------------------------------------------------
    ' Save
    Private Sub StoreReg()
        Dim regkey As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.CurrentUser.CreateSubKey(REGKEY_STR)

        If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
            If sPrintCtrl.GetMediaClassString() = MEDIA_CLASS_NAME_HDM Then
                regkey.SetValue("ICCTblFile820_PP", TextBox_ICCTblFilename.Text)
            Else
                regkey.SetValue("ICCTblFile820_SD", TextBox_ICCTblFilename.Text)
            End If
            regkey.SetValue("LUTFile820", TextBox_LUTFilename.Text)
        Else
            regkey.SetValue("ICCTblFile", TextBox_ICCTblFilename.Text)
            regkey.SetValue("LUTFile", TextBox_LUTFilename.Text)
        End If

        regkey.SetValue("CorrectFile", TextBox_CorrFilename.Text)
        regkey.SetValue("ImageFile", TextBox_SrcImgFilename.Text)
        regkey.SetValue("Resolution", ComboBox_Resolution.SelectedIndex)
        regkey.SetValue("HighDensity", ComboBox_HighDensity.SelectedIndex)
        regkey.SetValue("OverCoat", ComboBox_OverCoat.SelectedIndex)
        regkey.SetValue("NumOfDiv", CInt(Numeric_NumOfDiv.Value))
        regkey.SetValue("CorrectType", Radio_CorrSensorValue.Checked)

        regkey.Close()

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Select ICC Table.
    '* ------------------------------------------------------------------------------
    Private Function GetICCTableName(ByVal mediaclass As String, ByVal regload As Boolean) As String
        Dim regkey As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.CurrentUser.CreateSubKey(REGKEY_STR)
        Dim rtn As String

        If mediaclass = MEDIA_CLASS_NAME_HDM Then
            If regload = True Then rtn = regkey.GetValue("ICCTblFile820_PP")
            If IsNothing(rtn) = True Then
                rtn = "..\..\ParameterFiles\ICCTblData_0020_PP.csv"
            End If
        Else
            If regload = True Then rtn = regkey.GetValue("ICCTblFile820_SD")
            If IsNothing(rtn) = True Then
                rtn = "..\..\ParameterFiles\ICCTblData_0021_SD.csv"
            End If
        End If

        regkey.Close()

        Return rtn

    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Load Registry.
    '* ------------------------------------------------------------------------------
    Private Sub LoadReg()
        Dim regkey As Microsoft.Win32.RegistryKey = Microsoft.Win32.Registry.CurrentUser.CreateSubKey(REGKEY_STR)

        Try
            If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
                TextBox_ICCTblFilename.Text = GetICCTableName(sPrintCtrl.GetMediaClassString(), True)
                TextBox_LUTFilename.Text = regkey.GetValue("LUTFile820")
                If TextBox_LUTFilename.Text.Length = 0 = True Then
                    TextBox_LUTFilename.Text = "..\..\ParameterFiles\LUTData820_0010.csv"
                End If
            Else
                TextBox_ICCTblFilename.Text = regkey.GetValue("ICCTblFile")
                If TextBox_ICCTblFilename.Text.Length = 0 Then
                    TextBox_ICCTblFilename.Text = "..\..\ParameterFiles\ICCTblData_0010.csv"
                End If

                TextBox_LUTFilename.Text = regkey.GetValue("LUTFile")
                If TextBox_LUTFilename.Text.Length = 0 Then
                    TextBox_LUTFilename.Text = "..\..\ParameterFiles\LUTData_0010.csv"
                End If
            End If

            TextBox_CorrFilename.Text = regkey.GetValue("CorrectFile")
            If TextBox_CorrFilename.Text.Length = 0 Then
                TextBox_CorrFilename.Text = "..\..\ParameterFiles\CorrectTableData.csv"
            End If

            sImgFilename = regkey.GetValue("ImageFile")
            If sImgFilename.Length = 0 Then
                sImgFilename = "..\Images\Sample01.bmp"
            End If

            ComboBox_Resolution.SelectedIndex = regkey.GetValue("Resolution")

            If Label_PrinterName.Text.Contains(PRNTER_NAME_DS820) Then
                ComboBox_HighDensity.SelectedIndex = regkey.GetValue("HighDensity")
            Else
                ComboBox_HighDensity.SelectedIndex = 0
            End If

            ComboBox_OverCoat.SelectedIndex = regkey.GetValue("OverCoat")

        Catch ex As Exception
            If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
                TextBox_ICCTblFilename.Text = GetICCTableName(sPrintCtrl.GetMediaClassString(), False)
                TextBox_LUTFilename.Text = "..\..\ParameterFiles\LUTData820_0010.csv"
            Else
                TextBox_ICCTblFilename.Text = "..\..\ParameterFiles\ICCTblData_0010.csv"
                TextBox_LUTFilename.Text = "..\..\ParameterFiles\LUTData_0010.csv"
            End If

            TextBox_CorrFilename.Text = "..\..\ParameterFiles\CorrectTableData.csv"
            sImgFilename = "..\Images\Sample01.bmp"
            ComboBox_Resolution.SelectedIndex = 0
            ComboBox_HighDensity.SelectedIndex = 0
            ComboBox_OverCoat.SelectedIndex = 0
        End Try

        Try
            Numeric_NumOfDiv.Value = CInt(regkey.GetValue("NumOfDiv"))
        Catch ex As Exception
            Numeric_NumOfDiv.Value = Numeric_NumOfDiv.Minimum
        End Try

        If regkey.GetValue("CorrectType") Is Nothing Then
            Radio_CorrSensorValue.Checked = True
        Else
            Radio_CorrSensorValue.Checked = regkey.GetValue("CorrectType")
        End If

        Radio_CorrImmValue.Checked = Not Radio_CorrSensorValue.Checked

        regkey.Close()

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Add paper size
    '* ------------------------------------------------------------------------------
    Private Sub AddPaperSize()
        ComboBox1.Items.Clear()
        If PAPER_A5 <> -1 Then ComboBox1.Items.Add("6x8")
        If PAPER_8x10 <> -1 Then ComboBox1.Items.Add("8x10")
        If PAPER_8x12 <> -1 Then ComboBox1.Items.Add("8x12")
        If PAPER_A4 <> -1 Then ComboBox1.Items.Add("A4")
        If ComboBox1.Items.Count <> 0 Then
            ComboBox1.Text = ComboBox1.Items(0).ToString
        Else
            ComboBox1.Text = ""
        End If
    End Sub
    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Load the main form)
    '* ------------------------------------------------------------------------------
    Private Sub FormMain_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Initialize()

        ' Select printer.
        SelectPrinter()

        'Load the previous input values.
        LoadReg()

        ' Set the version string to title bar.
        Me.Text = Me.Text + "  (Ver." + My.Application.Info.Version.Major.ToString + "." _
        + My.Application.Info.Version.Minor.ToString + "." _
        + My.Application.Info.Version.Build.ToString + ")"

        ' Update property
        ChangeCtrlButtonProperty()
        ChangeImageCtrlProperty()

        'Debug
        TextBox_SrcImgFilename.Text = sImgFilename

        ' Add Paper Size
        AddPaperSize()

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Select "Menu - Select port")
    '* ------------------------------------------------------------------------------
    Private Sub PortSettingRToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles PortSettingRToolStripMenuItem.Click
        ' Select the connect printer.
        SelectPrinter()
        ChangeImageCtrlProperty(True)
        AddPaperSize()
        LoadReg()

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Select "Menu - Exit")
    '* ------------------------------------------------------------------------------
    Private Sub ExitxToolStripMenu_File_Exit_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ExitxToolStripMenu_File_Exit.Click
        ' Exit confirmation.
        If MessageBox.Show("Are you sure?", "Panorama Print Application", MessageBoxButtons.OKCancel, MessageBoxIcon.Question) <> MsgBoxResult.Ok Then
            Return
        End If

        'Store some information.
        StoreReg()

        Me.Close()

        ' Terminate this application.
        End

    End Sub
    Private Sub FormPanoramaApp_Disposed(ByVal sender As Object, ByVal e As System.EventArgs) Handles Me.Disposed
        If PanoEnv IsNot Nothing Then
            PanoEnv.Dispose()
            PanoEnv = Nothing
        End If

        ' Clear the previous image data.
        If sBmpSrc IsNot Nothing Then
            sBmpSrc.Dispose()
            sBmpSrc = Nothing
        End If

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Button "Open"(ICCTblFilename) clicked)
    '* ------------------------------------------------------------------------------
    Private Sub Button_ICCTblFilename_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_ICCTblFilename.Click
        Dim ofd As New OpenFileDialog()

        If TextBox_ICCTblFilename.Text.Length > 0 Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(TextBox_ICCTblFilename.Text)
            ofd.FileName = IO.Path.GetFileName(TextBox_ICCTblFilename.Text)
        End If
        ofd.Filter = "csv file(*.csv)|*.csv|All (*.*)|*.*"
        ofd.FilterIndex = 1
        ofd.Title = "Please select ICC table file."
        ofd.RestoreDirectory = True
        ofd.CheckFileExists = True
        ofd.CheckPathExists = True

        ' Show file dialog.
        If ofd.ShowDialog() = DialogResult.OK Then
            TextBox_ICCTblFilename.Text = ofd.FileName
        End If

        ofd.Dispose()
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Button "Open"(LUTTblFilename) clicked)
    '* ------------------------------------------------------------------------------
    Private Sub Button_LUTTblFilename_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_LUTTblFilename.Click
        Dim ofd As New OpenFileDialog()

        If TextBox_LUTFilename.Text.Length > 0 Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(TextBox_LUTFilename.Text)
            ofd.FileName = IO.Path.GetFileName(TextBox_LUTFilename.Text)
        End If
        ofd.Filter = "csv file(*.csv)|*.csv|All (*.*)|*.*"
        ofd.FilterIndex = 1
        ofd.Title = "Please select the lookup table file."
        ofd.RestoreDirectory = True
        ofd.CheckFileExists = True
        ofd.CheckPathExists = True

        ' Show file dialog.
        If ofd.ShowDialog() = DialogResult.OK Then
            TextBox_LUTFilename.Text = ofd.FileName
        End If

        ofd.Dispose()
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Button "Open"(LUTTblFilename) clicked)
    '* ------------------------------------------------------------------------------
    Private Sub Button_CorrTblFilename_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_CorrTblFilename.Click
        Dim ofd As New OpenFileDialog()

        If TextBox_CorrFilename.Text.Length > 0 Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(TextBox_CorrFilename.Text)
            ofd.FileName = IO.Path.GetFileName(TextBox_CorrFilename.Text)
        End If
        ofd.Filter = "csv file(*.csv)|*.csv|All (*.*)|*.*"
        ofd.FilterIndex = 1
        ofd.Title = "Please select the correction table file."
        ofd.RestoreDirectory = True
        ofd.CheckFileExists = True
        ofd.CheckPathExists = True

        ' Show file dialog.
        If ofd.ShowDialog() = DialogResult.OK Then
            TextBox_CorrFilename.Text = ofd.FileName
        End If

        ofd.Dispose()
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Button "Open"(SrcImgFilename) clicked)
    '* ------------------------------------------------------------------------------
    Private Sub Button_SrcImgFilename_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_SrcImgFilename.Click
        Dim ofd As New OpenFileDialog()

        'If a image file name has been registered, it will be set the its file path.
        If TextBox_SrcImgFilename.Text.Length > 0 Then
            ofd.InitialDirectory = IO.Path.GetDirectoryName(TextBox_SrcImgFilename.Text)
            ofd.FileName = IO.Path.GetFileName(TextBox_SrcImgFilename.Text)
        Else
            ofd.InitialDirectory = System.IO.Directory.GetCurrentDirectory()
            ofd.FileName = ""
        End If
        ofd.Filter = "Image file(*.jpg;*.bmp;*.png;*.tiff)|*.jpg;*.bmp;*.png;*.tiff|All (*.*)|*.*"
        ofd.FilterIndex = 1
        ofd.Title = "Please select an image file"
        ofd.RestoreDirectory = False
        ofd.CheckFileExists = True
        ofd.CheckPathExists = True

        ' Show file dialog.
        If ofd.ShowDialog() = DialogResult.OK Then
            TextBox_SrcImgFilename.Text = ofd.FileName
        End If

        ofd.Dispose()
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Number of the divide image was changed)
    '* ------------------------------------------------------------------------------
    Private Sub Numeric_NumOfDiv_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Numeric_NumOfDiv.ValueChanged, Numeric_CorrImmHumidVal.ValueChanged, Numeric_CorrImmTempVal.ValueChanged

        'Clear panorama image.
        ClearPanoramaImage()

        Update()

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Overlapping width was changed)
    '* ------------------------------------------------------------------------------
    Private Sub Numeric_OverlapWidth_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs)
        ClearPanoramaImage()
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Dropped in the main form)
    '* ------------------------------------------------------------------------------
    Private Sub FormMain_DragDrop(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragDrop
        Dim fileName As String() = CType(e.Data.GetData(DataFormats.FileDrop, False), String())

        ' Valid name?
        If fileName.Length > 0 Then
            TextBox_SrcImgFilename.Text = fileName(0)
        End If

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Dragged in the main form)
    '* ------------------------------------------------------------------------------
    Private Sub FormMain_DragEnter(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DragEventArgs) Handles MyBase.DragEnter

        ' Is it validity?
        If e.Data.GetDataPresent(DataFormats.FileDrop) Then
            e.Effect = DragDropEffects.Copy
        Else
            e.Effect = DragDropEffects.None
        End If

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Changed the input text of source image file)
    '* ------------------------------------------------------------------------------
    Private Sub TextBox_SrcImgFilename_TextChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles TextBox_SrcImgFilename.TextChanged

        'Confirm this file name exists.
        If IO.File.Exists(TextBox_SrcImgFilename.Text) Then
            LoadSourceFile()
        End If

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Closed the main form)
    '* ------------------------------------------------------------------------------
    Private Sub FormMain_FormClosed(ByVal sender As System.Object, ByVal e As System.Windows.Forms.FormClosedEventArgs) Handles MyBase.FormClosed
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Waiting Transition Printer Status
    '* ------------------------------------------------------------------------------
    Private Function WaitingStatusTransition() As Boolean
        Dim stat As Integer

        ' Waiting until changing to the printing status
        While PrgEnv.GetStatus(sPortNo) <> STATUS_USUALLY_PRINTING
            If xPrint_Cancel = True Then Return False
            Sleep(1000)
            System.Windows.Forms.Application.DoEvents()
        End While

        ' Waiting until changing to the idle status
        While PrgEnv.GetStatus(sPortNo) = STATUS_USUALLY_PRINTING
            If xPrint_Cancel = True Then Return False
            Sleep(1000)
            System.Windows.Forms.Application.DoEvents()
        End While

        'If the printer status is abnormal, then it will exit process.
        stat = PrgEnv.GetStatus(sPortNo)
        If stat <> STATUS_USUALLY_IDLE And _
           stat <> STATUS_USUALLY_COOLING And _
           stat <> STATUS_USUALLY_MOTCOOLING Then
            Return False
        End If

        Return True

    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Set paper size information
    '* ------------------------------------------------------------------------------
    Private Function SetPaperSizeInformation() As Boolean
        Dim sts As Boolean = True

        Select Case ComboBox1.Text
            Case "6x8"
                PAPER_SIZE_WIDTH = PAPER_SIZE_WIDTH_6x8
                PAPER_SIZE_HEIGHT = PAPER_SIZE_HEIGHT_6x8
                sPrintCtrl.SetPprSize(PAPER_A5)
            Case "8x10"
                PAPER_SIZE_WIDTH = PAPER_SIZE_WIDTH_8x10
                PAPER_SIZE_HEIGHT = PAPER_SIZE_HEIGHT_8x10
                sPrintCtrl.SetPprSize(PAPER_8x10)
            Case "8x12"
                PAPER_SIZE_WIDTH = PAPER_SIZE_WIDTH_8x12
                PAPER_SIZE_HEIGHT = PAPER_SIZE_HEIGHT_8x12
                sPrintCtrl.SetPprSize(PAPER_8x12)
            Case "A4"
                PAPER_SIZE_WIDTH = PAPER_SIZE_WIDTH_A4
                PAPER_SIZE_HEIGHT = PAPER_SIZE_HEIGHT_A4
                sPrintCtrl.SetPprSize(PAPER_A4)
            Case Else
                sts = False
        End Select

        Return sts

    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. ("Panorama Print" Button click)
    '* ------------------------------------------------------------------------------
    Private Sub Button_ProcAndPrint_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button_ProcAndPrint.Click
        Dim rnum As Integer = CopyUpDown.Value

        'Set paper size information
        If SetPaperSizeInformation() = False Then Return

        ChangeImageCtrlProperty(False)

        Button_ProcAndPrint.Visible = False
        xPrint_Cancel = False

        ' Repeat the designated number of print
        While rnum > 0

            ' Check printer condition.
            If Not CheckPanoramaEnable(True) Then
                ' Printer is unhealthy.
                Exit While
            End If

            ' Call function to divide the source image data.
            If DevidePanoramaImage() < 0 Then
                Exit While
            End If

            ' Call function to make the panorama image data.
            If MakePanoramaImage() = False Then
                Exit While
            End If

            'Wating printer status transition <Printing -> Idle>
            If WaitingStatusTransition() = False Then
                Exit While
            End If

            rnum -= 1

        End While

        ' Update the button setting.
        ChangeCtrlButtonProperty()

        ChangeImageCtrlProperty(True)

        Button_ProcAndPrint.Visible = True

        Return

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. (Radio button of correction function)
    '* ------------------------------------------------------------------------------
    Private Sub Radio_CorrSensorValue_CheckedChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Radio_CorrSensorValue.CheckedChanged, Radio_CorrImmValue.CheckedChanged

        UpdateEnableCorrectionObj()
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Set enable config of the sensor value label
    '* ------------------------------------------------------------------------------
    Private Sub UpdateEnableCorrectionObj()

        If CheckBox_CorrectEnable Then
            ' Set enabled
            Label_CorrSensTempVal.Enabled = Radio_CorrSensorValue.Checked
            Label_CorrSensHumidVal.Enabled = Radio_CorrSensorValue.Checked
            Numeric_CorrImmTempVal.Enabled = Radio_CorrImmValue.Checked
            Numeric_CorrImmHumidVal.Enabled = Radio_CorrImmValue.Checked

            If Radio_CorrSensorValue.Checked Then
                Label_CorrSensTempVal.BackColor = SystemColors.HighlightText
                Label_CorrSensHumidVal.BackColor = SystemColors.HighlightText
            Else
                Label_CorrSensTempVal.BackColor = SystemColors.Control
                Label_CorrSensHumidVal.BackColor = SystemColors.Control
                Label_CorrSensTempVal.Text = "--"
                Label_CorrSensHumidVal.Text = "--"
            End If
        Else
            Label_CorrSensTempVal.BackColor = SystemColors.Control
            Label_CorrSensHumidVal.BackColor = SystemColors.Control
            Label_CorrSensTempVal.Text = "--"
            Label_CorrSensHumidVal.Text = "--"
        End If

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Timer event handler. 
    '* ------------------------------------------------------------------------------
    Private Sub Timer_Mon_Tick(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Timer_Mon.Tick
        UpdateSensorInfomation()
        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Read sensor infomation from printer.
    '* ------------------------------------------------------------------------------
    Private Sub UpdateSensorInfomation()
        Dim temp As Single
        Dim humid As Single

        If sPrintCtrl.IsConnected() Then
            'Update the connected printer status.
            Label_PrinterSts.Text = sPrintCtrl.GetStatusString()
            If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
                Label_MediaClass.Text = sPrintCtrl.GetMediaClassString()
            Else
                Label_MediaClass.Text = MEDIA_CLASS_NAME_HQL    'HQL Only
            End If

            If CheckBox_CorrectEnable Then
                'Update the sensor infomation of the connected printer.
                If Radio_CorrSensorValue.Checked And sPrintCtrl.GetSensorValue(temp, humid) Then
                    Label_CorrSensTempVal.Text = temp.ToString("F0")
                    Label_CorrSensHumidVal.Text = humid.ToString("F0")
                Else
                    Label_CorrSensTempVal.Text = "--"
                    Label_CorrSensHumidVal.Text = "--"
                End If
            End If
        End If
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Update the form control propertys.
    '* ------------------------------------------------------------------------------
    ' During standby.
    Private Sub ChangeCtrlButtonProperty(Optional ByVal bEnable As Boolean = True)
        Dim bEnableProc As Boolean = False
        Dim bEnablePrint As Boolean = False

        If bEnable = True Then

            ' Has source bitmap?
            If sBmpSrc IsNot Nothing Then
                bEnableProc = True
            End If

            'Printable state?
            If sBmpPanoramaTbl IsNot Nothing Then
                If sPrintCtrl.IsConnected() Then
                    bEnablePrint = True
                End If
            End If
        Else
            bEnableProc = False
            bEnablePrint = False
        End If

        'Set properties.
        Button_ProcAndPrint.Enabled = bEnableProc And sPrintCtrl.IsConnected()

        Return
    End Sub

    ' Force update.
    Private Sub ChangeImageCtrlProperty(Optional ByVal bEnable As Boolean = True)
        ChangeCtrlButtonProperty(bEnable)

        ComboBox_Resolution.Enabled = bEnable

        If Label_PrinterName.Text.Contains(PRNTER_NAME_DS820) Then
            If ComboBox_Resolution.Text.Contains("600") Then
                ComboBox_HighDensity.Enabled = bEnable
            Else
                ComboBox_HighDensity.Enabled = False
            End If
        Else
            ComboBox_HighDensity.Enabled = False
        End If

        ComboBox_OverCoat.Enabled = bEnable
        Panel_SourceFile.Enabled = bEnable
        GroupBox_ParameterFiles.Enabled = bEnable
        GroupBox_CorrType.Enabled = bEnable
        CopyUpDown.Enabled = bEnable
        ComboBox1.Enabled = bEnable
        Button1.Enabled = bEnable
        Numeric_NumOfDiv.Enabled = bEnable

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Initialize API
    '* ------------------------------------------------------------------------------
    Private Sub InitializeAPI()

        If PrgEnv IsNot Nothing Then PrgEnv.Dispose()
        If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
            If CheckEnvironment() = 32 Then
                PrgEnv = New ClassForDS820_x86
            Else
                PrgEnv = New ClassForDS820_x64
            End If
        Else
            If CheckEnvironment() = 32 Then
                PrgEnv = New ClassForDS620_x86
            Else
                PrgEnv = New ClassForDS620_x64
            End If
        End If

        'Change Over coat setting
        ComboBox_OverCoat.Items.Clear()
        ComboBox_OverCoat.Items.Add("Glossy")
        ComboBox_OverCoat.Items.Add("Matte")
        ComboBox_OverCoat.Items.Add("Luster")
        If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
            ComboBox_OverCoat.Items.Add("Light Matte")
            ComboBox_OverCoat.Items.Add("Fine Matte")
        End If

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Show the select printer dialog.
    '* ------------------------------------------------------------------------------
    Private Sub SelectPrinter()
        ' Select printer. Show the select dialog.
        If DlgPortSelecter.ShowDialog() = Windows.Forms.DialogResult.OK Then

            InitializeAPI()     'Initialize API
            sPrintCtrl.OpenPort(DlgPortSelecter.PrinterName(), DlgPortSelecter.PortName())            'Set the printer names.

        End If

        If sPrintCtrl.IsConnected() Then
            Label_PrinterName.Text = DlgPortSelecter.PrinterName()
            Timer_Mon.Enabled = True
        Else
            Label_PrinterName.Text = "Not connected"
            Timer_Mon.Enabled = False
        End If

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Adjust resolution
    '* ------------------------------------------------------------------------------
    Private Sub AdjustResolution(ByRef bmp As Bitmap)
        Dim rHor As Single = bmp.HorizontalResolution
        Dim rVer As Single = bmp.VerticalResolution

        ' if the resolution of the original image is lager than 600dpi, it will be set 600dpi.
        If rHor > RESOLUTION_LIMIT Or rVer > RESOLUTION_LIMIT Then
            Dim ratioWidth As Single
            Dim ratioHeight As Single
            Dim gc As Graphics

            ratioWidth = (RESOLUTION_LIMIT / rHor) * bmp.Width
            ratioWidth -= (ratioWidth Mod 4)
            ratioHeight = (ratioWidth / bmp.Width) * bmp.Height

            Dim canvas = New Bitmap(CInt(ratioWidth), CInt(ratioHeight))
            canvas.SetResolution(RESOLUTION_LIMIT, RESOLUTION_LIMIT)

            gc = Graphics.FromImage(canvas)
            gc.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
            gc.DrawImage(bmp, 0, 0, canvas.Width, canvas.Height)
            gc.Dispose()

            bmp.Dispose()
            bmp = canvas
        End If

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Load the source image file.
    '* ------------------------------------------------------------------------------
    Private Sub LoadSourceFile()
        Dim bmpTmp As Bitmap
        Dim rect As Rectangle

        Try
            ' Clear the previous image data.
            If sBmpSrc IsNot Nothing Then
                sBmpSrc.Dispose()
                sBmpSrc = Nothing
            End If

            ' Load image data from file.)
            bmpTmp = Image.FromFile(TextBox_SrcImgFilename.Text)

            ' Change pixel format.
            rect.X = 0
            rect.Y = 0
            rect.Width = bmpTmp.Width - (bmpTmp.Width Mod 4)
            rect.Height = bmpTmp.Height
            sBmpSrc = bmpTmp.Clone(rect, Imaging.PixelFormat.Format32bppRgb)
            If bmpTmp.Width < bmpTmp.Height Then
                sBmpSrc.RotateFlip(RotateFlipType.Rotate270FlipNone)
            End If
            bmpTmp.Dispose()

            ' Adjust resolution
            AdjustResolution(sBmpSrc)

        Catch ex As Exception
            ' Error
            MessageBox.Show("Image file read failure." + ex.ToString, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)

            ' Clear the previous image data.
            If sBmpSrc IsNot Nothing Then
                sBmpSrc.Dispose()
                sBmpSrc = Nothing
            End If
        End Try

        ' Update the source picture.
        ImgShowPicture(PictureBox_SrcImg, sBmpSrc)

        ' Update property of buttons.
        ChangeCtrlButtonProperty()

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Clear panorama image data.
    '* ------------------------------------------------------------------------------
    Private Sub ClearPanoramaImage(Optional ByVal bUpdateButton As Boolean = True)
        If sBmpPanoramaTbl IsNot Nothing Then

            For i = 0 To (sBmpPanoramaTbl.Length - 1)
                If sBmpPanoramaTbl(i) IsNot Nothing Then sBmpPanoramaTbl(i).Dispose()
            Next
            sBmpPanoramaTbl = Nothing
            If bUpdateButton Then
                ChangeCtrlButtonProperty()
            End If
        End If
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Prepare the printable panorama images.
    '* ------------------------------------------------------------------------------
    Private Function DevidePanoramaImage() As Integer
        Dim res As Integer
        Dim divParam As StructDivideParam

        '*! ---------------------------------------
        '*	Clear the previous data.
        '*  ---------------------------------------
        ClearPanoramaImage(False)

        '*! ---------------------------------------
        '*	Divide the target image.
        '*  ---------------------------------------

        'Allocate buffer.
        Pano_AllocBitmap(sBmpPanoramaTbl, PAPER_SIZE_WIDTH, PAPER_SIZE_HEIGHT, Numeric_NumOfDiv.Value)

        ' Set parameters of the divide condition.
        divParam.numOfDiv = Numeric_NumOfDiv.Value
        divParam.overlapWidth = CInt(Numeric_OverlapWidth * 300.0)  ' Convert inch to pixel.
        divParam.srcWidth = sBmpSrc.Width
        divParam.srcHeight = sBmpSrc.Height
        divParam.dstWidth = PAPER_SIZE_WIDTH
        divParam.dstHeight = PAPER_SIZE_HEIGHT

        ' Allocate the distination buffers.

        res = Pano_DivImage(sBmpPanoramaTbl, sBmpSrc, divParam)

        If res < 0 Then
            ErrorMessageOut(res)
        End If

        Return res

    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Waiting for receive buffer.
    '* ------------------------------------------------------------------------------
    Private Function WaitingForReceiveBuffer() As Boolean
        Do
            If xPrint_Cancel = True Then
                Return False
            End If

            If sPrintCtrl.CheckFreeBuffer() > 0 Then
                Exit Do
            End If
            System.Windows.Forms.Application.DoEvents()
            Sleep(1000)
        Loop

        Return True
    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Set environment correction.
    '* ------------------------------------------------------------------------------
    Private Function SetEnvironmentCorrection() As Integer
        Dim res As Integer = NORMAL
        Dim printerTemp As Double
        Dim printerHumid As Double

        If CheckBox_CorrectEnable Then

            ' Get the environment value.
            If Radio_CorrSensorValue.Checked Then
                'Read sensor value.
                UpdateSensorInfomation()

                Try
                    printerTemp = CDbl(Label_CorrSensTempVal.Text)
                    printerHumid = CDbl(Label_CorrSensHumidVal.Text)
                Catch ex As Exception
                    ErrorMessageOut(ERR_INTERNAL_PROC)
                    Return False
                End Try

            Else
                printerTemp = Numeric_CorrImmTempVal.Value
                printerHumid = Numeric_CorrImmHumidVal.Value
            End If

            Dim stCurrentDir As String = System.IO.Directory.GetCurrentDirectory()

            ' Set the correction parameter to the panorama module.
            res = Pano_SetCorrectionParam(printerTemp, printerHumid, TextBox_CorrFilename.Text)

        End If

        Return res
    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Process the printable panorama images.
    '* ------------------------------------------------------------------------------
    Private Function MakePanoramaImage() As Boolean
        Dim i As Integer
        Dim res As Integer
        Dim bRes As Boolean
        Dim bLast As Boolean = False
        Dim imgParam As StructPanoramaImageParam
        Dim iccTblFilename As String

        '*! ---------------------------------------
        '*	Prepare the correct function.
        '*  ---------------------------------------
        If TextBox_ICCTblFilename.Text.Length > 0 Then
            ' Enable ICC convert.
            iccTblFilename = TextBox_ICCTblFilename.Text
        Else
            ' Disable ICC convert.
            iccTblFilename = Nothing
        End If

        ' Reset the panorama module.
        PanoEnv.PP_Reset()

        'Environment Correction 
        res = SetEnvironmentCorrection()
        If res < 0 Then
            GoTo _EndFunction
        End If

        '*! ---------------------------------------
        '*	Process the divided image.
        '*  ---------------------------------------
        imgParam.overlapWidth = CInt(Numeric_OverlapWidth * 300.0)    ' Convert inch to pixel.
        imgParam.srcWidth = PAPER_SIZE_WIDTH
        imgParam.srcHeight = PAPER_SIZE_HEIGHT

        ' Set print settings.
        PrintSetting()

        ' Processed in the order to be print.
        For i = (Numeric_NumOfDiv.Value - 1) To 0 Step (-1)

            If i = (Numeric_NumOfDiv.Value - 1) Then
                ' The 1st position.
                res = Pano_ProcImage1st(sBmpPanoramaTbl(i), imgParam, iccTblFilename, TextBox_LUTFilename.Text)
                If res < 0 Then Exit For

            ElseIf i <> 0 Then
                ' The middle position.
                res = Pano_ProcImageMid(sBmpPanoramaTbl(i), imgParam, iccTblFilename, TextBox_LUTFilename.Text)
                If res < 0 Then Exit For

            Else
                ' The last position.
                res = Pano_ProcImageLast(sBmpPanoramaTbl(i), imgParam, iccTblFilename, TextBox_LUTFilename.Text)
                If res < 0 Then Exit For

                bLast = True    'last image
            End If

            'Wait for receive buffer
            If WaitingForReceiveBuffer() = False Then
                Exit For
            End If

            'Send panoramic prints settings
            bRes = sPrintCtrl.SetPanoramaSetting(bLast, CSng(Numeric_OverlapWidth))

            'Print Start
            If bRes Then
                sPrintCtrl.DoPrint(sBmpPanoramaTbl(i))
            Else
                Exit For
            End If

        Next

_EndFunction:
        If res < 0 Then
            ' Failure
            ErrorMessageOut(res)
            Return False
        End If

        Return True
    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Display error messegae.
    '* ------------------------------------------------------------------------------
    Public Sub ErrorMessageOut(ByVal res As Integer)
        Dim msg As String = "Failed in panorama image process. " + vbCrLf  '"(Code " + res.ToString + ")"
        Select Case res
            Case ERR_FILE_OPEN             ' Failed to file open.
                msg += "<< " + "File open error." + " >>"
            Case ERR_NO_MEMORY             ' Free memory is empty.
                msg += "<< " + "Memory not enough." + " >>"
            Case ERR_PARAM                 ' Invalid parameter.
                msg += "<< " + "Invalid parameter." + " >>"
            Case ERR_OBJ                   ' State is invalid.
                msg += "<< " + "Invalid state." + " >>"
            Case ERR_NO_DATA               ' No data.
                msg += "<< " + "No data." + " >>"
            Case ERR_INTERNAL_PARAM        ' Invalid internal parameter.
                msg += "<< " + "Invalid internal parameter." + " >>"
            Case ERR_INTERNAL_PROC         ' Internal error.
                msg += "<< " + "Internal error." + " >>"
            Case ERR_NO_SUPPORT            ' Unsupported.
                msg += "<< " + "Unsupported." + " >>"
            Case ERR_INTERNAL_NO_DUTY      ' No duty data.
                msg += "<< " + "No duty data." + " >>"
            Case ERR_INVALID_ENVIRONMENT   ' No duty data.
                msg += "<< " + "Invalid Environment." + " >>"

            Case ERR_ICCTBLFILE_OPEN       ' Failed to ICC file open.
                msg += "<< " + "Invalid ICC table management file name or path." + " >>"
            Case ERR_ICCTBLFILE_NODATA     ' Failed to ICC file open.
                msg += "<< " + "ICC table has no data." + " >>"
            Case ERR_ICCTBLFILE_FORMAT     ' Failed to ICC file open.
                msg += "<< " + "Invalid ICC table file format." + " >>"
            Case ERR_ICCPROFILE_OPEN       ' Failed to ICC file open.
                msg += "<< " + "ICC profile open error." + " >>"
            Case ERR_ICCPROFILE_INIT       ' Error in Create convert object of ICM.
                msg += "<< " + "Creating color transform has failed." + " >>"
            Case ERR_ICCPROFILE_PROC       ' ICM conversion failure.
                msg += "<< " + "Color conversion has failed." + " >>"

            Case ERR_LUTFILE_OPEN          ' Failed to Look-up table file open.
                msg += "<< " + "Invalid Look-Up Table file name or path." + " >>"
            Case ERR_LUTFILE_NODATA        ' No data.
                msg += "<< " + "Look-Up Table file has no data." + " >>"
            Case ERR_LUTFILE_FORMAT        ' File format error of LUT file.
                msg += "<< " + "Invalid Look-Up Table file format." + " >>"
            Case ERR_LUTFILE_ELM_TOOSMALL  ' Element number of LUT file is too small.
                msg += "<< " + "The element number of Look-Up Table file is too little." + " >>"

            Case ERR_CORRFILE_OPEN         ' Failed to Correction table file open.
                msg += "<< " + "Invalid correct table file name or path." + " >>"
            Case ERR_CORRFILE_NODATA       ' No data.
                msg += "<< " + "Correct table file has no data." + " >>"
            Case ERR_CORRFILE_FORMAT       ' File format error of the Correction table file.
                msg += "<< " + "Invalid correct table file format." + " >>"
            Case ERR_CORRFILE_DUPLICATE    'Duplicate data.
                msg += "<< " + "Correct table file data has overlapped." + " >>"
            Case Else
                Return
        End Select

        MessageBox.Show(msg, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error)

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Printer config.
    '* ------------------------------------------------------------------------------
    Private Sub PrintSetting()

        ' Resolution
        If ComboBox_Resolution.SelectedIndex = 0 Then
            '300dpi
            sPrintCtrl.SetResolution(300)
        Else
            '600dpi
            sPrintCtrl.SetResolution(600)
        End If

        ' High density mode
        If ComboBox_HighDensity.SelectedIndex = 0 Then
            'Disable
            sPrintCtrl.SetHighDensityMode(DM_HIGH_DENSITY_DISABLE)
        Else
            'Enable
            sPrintCtrl.SetHighDensityMode(DM_HIGH_DENSITY_ENABLE)
        End If

        'Over coat
        If ComboBox_OverCoat.SelectedIndex = 0 Then
            'Glossy
            sPrintCtrl.SetOvercoat(DM_OVERCOAT_GLOSSY)
        ElseIf ComboBox_OverCoat.SelectedIndex = 1 Then
            'Matte
            sPrintCtrl.SetOvercoat(DM_OVERCOAT_MATTE)
        ElseIf ComboBox_OverCoat.SelectedIndex = 2 Then
            'LUSTER
            sPrintCtrl.SetOvercoat(DM_OVERCOAT_LUSTER)
        ElseIf ComboBox_OverCoat.SelectedIndex = 3 Then
            'Light Matte
            sPrintCtrl.SetOvercoat(DM_OVERCOAT_LIGHT_MATTE)
        ElseIf ComboBox_OverCoat.SelectedIndex = 4 Then
            'Fine Matte
            sPrintCtrl.SetOvercoat(DM_OVERCOAT_FINE)
        Else
            'Glossy
            sPrintCtrl.SetOvercoat(DM_OVERCOAT_GLOSSY)
        End If

        Return
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Confirm able to the panorama printing.
    '* ------------------------------------------------------------------------------
    Private Function CheckPanoramaEnable(ByVal wait As Boolean)
        Dim numOfRemain As Integer
        Dim retcode As String

        'Check whether the media remaining is enough.
        numOfRemain = sPrintCtrl.GetMediaRemainCount()
        If numOfRemain < Numeric_NumOfDiv.Value Then
            MessageBox.Show("Media remaining number is not enough.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
            Return False
        End If

        'Check whether the panorama printing is possible.
        If wait = True Then
            While Not PRINTER_INFO_PanoramaPrint(retcode)
                If xPrint_Cancel = True Then Return False
                Sleep(1000)
                System.Windows.Forms.Application.DoEvents()
            End While
        Else
            If Not PRINTER_INFO_PanoramaPrint(retcode) Then
                MessageBox.Show("Can not print the panoramic printing, now.", "Warning", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
                Return False
            End If
        End If

        Return True
    End Function

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. ("Cancel" button Clicked)
    '* ------------------------------------------------------------------------------
    Private Sub Button2_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles Button2.Click
        Button_ProcAndPrint.Visible = True
        xPrint_Cancel = True
    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. ("Initial Settings" Menu Clicked)
    '* ------------------------------------------------------------------------------
    Private Sub InitializeSettingsToolStripMenuItem_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles InitializeSettingsToolStripMenuItem.Click

        If DlgPortSelecter.PrinterName().Contains(PRNTER_NAME_DS820) Then
            TextBox_ICCTblFilename.Text = GetICCTableName(sPrintCtrl.GetMediaClassString(), False)
            TextBox_LUTFilename.Text = "..\..\ParameterFiles\LUTData820_0010.csv"
        Else
            TextBox_ICCTblFilename.Text = "..\..\ParameterFiles\ICCTblData_0010.csv"
            TextBox_LUTFilename.Text = "..\..\ParameterFiles\LUTData_0010.csv"
        End If

        TextBox_CorrFilename.Text = "..\..\ParameterFiles\CorrectTableData.csv"
        sImgFilename = "..\Images\Sample01.bmp"
        TextBox_SrcImgFilename.Text = sImgFilename
        ComboBox_Resolution.SelectedIndex = 0
        ComboBox_HighDensity.SelectedIndex = 0
        ComboBox_OverCoat.SelectedIndex = 0
        If ComboBox1.SelectedIndex <> -1 Then ComboBox1.SelectedIndex = 0
        CopyUpDown.Value = 1
        Numeric_NumOfDiv.Value = Numeric_NumOfDiv.Minimum
        Numeric_OverlapWidth = 2.0
        CheckBox_CorrectEnable = True
        Radio_CorrSensorValue.Checked = True

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Event handler. ("Resolution" ComboBox Changed)
    '* ------------------------------------------------------------------------------
    Private Sub ComboBox_Resolution_SelectedIndexChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles ComboBox_Resolution.SelectedIndexChanged

        If Label_PrinterName.Text.Contains(PRNTER_NAME_DS820) Then
            If ComboBox_Resolution.Text.Contains("600") Then
                ComboBox_HighDensity.Enabled = True
            Else
                ComboBox_HighDensity.Enabled = False
                ComboBox_HighDensity.Text = "Disable"
            End If
        End If

    End Sub

    '*! -----------------------------------------------------------------------------
    '*	@brief	:	Check 32 or 64 bit operating environment
    '* ------------------------------------------------------------------------------
    Public Function CheckEnvironment() As Integer

        'Get the information of the operating system
        Dim os As System.OperatingSystem = System.Environment.OSVersion

        'Investigate whether Windows NT system or not.
        If os.Platform = PlatformID.Win32NT Then
            OS_Version = os.Version.Major
        Else
            OS_Version = OSV_NotCompatible
        End If

        If IntPtr.Size = 4 Then
            Return 32
        Else
            Return 64
        End If

    End Function



End Class