﻿Imports System
Imports System.Windows
Imports System.Windows.Forms
Imports System.Drawing

Module ImageModule

	' Show bitmap data on the picture-box item.
	Public Sub ImgShowPicture(ByRef picBox As PictureBox, ByRef bmp As Bitmap)
		Dim canvas As New Bitmap(picBox.Width, picBox.Height)

		If bmp IsNot Nothing Then
			canvas = New Bitmap(picBox.Width, picBox.Height)

            ' Scaling & Trimming
			ImgAdjustSize(canvas, bmp)

            ' Display to PictureBox1
            picBox.Image = canvas
        Else
            picBox.Image = Nothing
		End If

	End Sub

	' Show bitmap data on the picture-box item.
	Public Sub ImgShowPictureFit(ByRef picBox As PictureBox, ByRef bmp As Bitmap)
		Dim canvas As New Bitmap(picBox.Width, picBox.Height)
		Dim gc As Graphics


		If bmp IsNot Nothing Then
			canvas = New Bitmap(picBox.Width, picBox.Height)

			gc = Graphics.FromImage(canvas)

			gc.DrawImage(bmp, 0, 0, picBox.Width, picBox.Height)

			gc.Dispose()

            'Display to PictureBox1
            picBox.Image = canvas
        Else
            picBox.Image = Nothing
		End If

	End Sub

	' Scaling & trimming bitmap image.
	Public Sub ImgAdjustSize(ByRef bmpDst As Bitmap, ByRef bmpSrc As Bitmap)
		Dim ratio As Single
		Dim ratioWidth As Single
		Dim ratioHeight As Single
		Dim gc As Graphics
		Dim canvas As Bitmap
		Dim rectSrc, rectDst As Rectangle

		'Get scale ratio.
        ratioWidth = CDbl(bmpDst.Width) / CDbl(bmpSrc.Width)
		ratioHeight = CDbl(bmpDst.Height) / CDbl(bmpSrc.Height)
		If ratioWidth > ratioHeight Then
			ratio = ratioWidth
		Else
			ratio = ratioHeight
		End If

		' Scaling
        canvas = New Bitmap(CInt(ratio * bmpSrc.Width), CInt(ratio * bmpSrc.Height))

        gc = Graphics.FromImage(canvas)
        gc.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
		gc.DrawImage(bmpSrc, 0, 0, canvas.Width, canvas.Height)
		gc.Dispose()

		'Triming
		rectDst.X = 0
		rectDst.Y = 0
		rectDst.Width = bmpDst.Width
		rectDst.Height = bmpDst.Height

		rectSrc.X = (canvas.Width - bmpDst.Width) / 2
		rectSrc.Y = (canvas.Height - bmpDst.Height) / 2
		rectSrc.Width = bmpDst.Width
		rectSrc.Height = bmpDst.Height

		gc = Graphics.FromImage(bmpDst)
		gc.DrawImage(canvas, rectDst, rectSrc, GraphicsUnit.Pixel)
        gc.Dispose()

        canvas.Dispose()

	End Sub

	' Rotate bitmap image 180 degrees. 
	Public Sub ImgRotete180deg(ByRef bmpSrc As Bitmap)
		bmpSrc.RotateFlip(RotateFlipType.Rotate180FlipNone)
	End Sub


	Public Sub ImgTrim(ByRef bmpDst As Bitmap, ByRef bmpSrc As Bitmap, ByRef rectSrc As Rectangle)
		Dim gc As Graphics
		Dim rectDst As Rectangle


		If bmpDst IsNot Nothing Then
			bmpDst.Dispose()
		End If

		bmpDst = New Bitmap(rectSrc.Width, rectSrc.Height)
		rectDst.X = 0
		rectDst.Y = 0
		rectDst.Width = rectSrc.Width
		rectDst.Height = rectSrc.Height

		gc = Graphics.FromImage(bmpDst)
		gc.DrawImage(bmpSrc, rectDst, rectSrc, GraphicsUnit.Pixel)
		gc.Dispose()

	End Sub


	Public Sub ImgHorizontalOffset(ByRef bmpSrc As Bitmap, ByVal offset As Integer)
		Dim gc As Graphics
		Dim dstRect As RectangleF
		Dim srcRect As RectangleF
		Dim tmpBmp = New Bitmap(bmpSrc.Width, bmpSrc.Height)


		If offset = 0 Then Return

		' Clear
		FillBitmap(tmpBmp, Color.White)

        ' Get the output-area and draw object
		gc = Graphics.FromImage(tmpBmp)

        ' Cropping
		If offset > 0 Then

			dstRect.X = offset
			dstRect.Y = 0
			dstRect.Width = bmpSrc.Width - offset
			dstRect.Height = bmpSrc.Height

			srcRect.X = 0
			srcRect.Y = 0
			srcRect.Width = bmpSrc.Width - offset
			srcRect.Height = bmpSrc.Height

		Else ' offset < 0
			dstRect.X = 0
			dstRect.Y = 0
			dstRect.Width = bmpSrc.Width + offset
			dstRect.Height = bmpSrc.Height

			srcRect.X = offset * -1
			srcRect.Y = 0
			srcRect.Width = bmpSrc.Width + offset
			srcRect.Height = bmpSrc.Height
		End If

        ' Copy the bmp-data and shifts it.
		gc.DrawImage(bmpSrc, dstRect, srcRect, GraphicsUnit.Pixel)

		gc.Dispose()

        ' Return the result file to the source buffer
        If bmpSrc IsNot Nothing Then
            bmpSrc.Dispose()
        End If
        bmpSrc = tmpBmp.Clone

	End Sub

End Module
