﻿Option Strict Off
Option Explicit On
Imports System.Runtime.InteropServices
Imports System.Drawing.Imaging
Imports PanoramaPrintSample.ClassPanoramaCommon

Module PanoramaModule

    '*! -----------------------------------------------------------------------------
    '*	@section	: External library(DLL) interface definitions.
    '* ------------------------------------------------------------------------------
    Public PanoEnv As Object

    '*! -----------------------------------------------------------------------------
    '*	@section	: Constant values.
    '* ------------------------------------------------------------------------------
    Const DIV_NUM_MAX As Integer = (3)

    '*! -----------------------------------------------------------------------------
    '*	@section	: Static variables.
    '* ------------------------------------------------------------------------------

    '*! -----------------------------------------------------------------------------
    '*	@brief	: Allocate bitmap buffer
    '*	@param	:
    '*	@return	: None
    '* ------------------------------------------------------------------------------
    Public Sub Pano_AllocBitmap(ByRef bmpDiv() As Bitmap, ByVal width As Integer, ByVal height As Integer, ByVal num As Integer)
        Dim i As Integer

        ReDim bmpDiv(num - 1)

        For i = 0 To (num - 1)
            bmpDiv(i) = New Bitmap(width, height, PixelFormat.Format32bppRgb)
        Next

    End Sub

	'*! -----------------------------------------------------------------------------
	'*	@brief	: Divide image
	'*	@param	:
	'*	@return	: >= 0 : Success	< 0: Failure
	'* ------------------------------------------------------------------------------
    Public Function Pano_DivImage(ByRef bmpDiv() As Bitmap, ByRef bmpSrc As Bitmap, ByRef parameter As StructDivideParam) As Integer
        Dim res As Integer
        Dim bmpaccSrc As BitmapAccess = New BitmapAccess
        Dim bmpaccDst0 As BitmapAccess = New BitmapAccess
        Dim bmpaccDst1 As BitmapAccess = New BitmapAccess
        Dim bmpaccDst2 As BitmapAccess = New BitmapAccess
        Dim datSrc(0) As Byte
        Dim datDst0(0) As Byte
        Dim datDst1(0) As Byte
        Dim datDst2(0) As Byte

        ' Get the image data object.
        datSrc = bmpaccSrc.LoadRawData(bmpSrc)

        If (parameter.numOfDiv > 0) Then
            datDst0 = bmpaccDst0.LoadRawData(bmpDiv(0))
        End If

        If (parameter.numOfDiv > 1) Then
            datDst1 = bmpaccDst1.LoadRawData(bmpDiv(1))
        End If

        If (parameter.numOfDiv > 2) Then
            datDst2 = bmpaccDst2.LoadRawData(bmpDiv(2))
        End If

        ' Call the divide function in the PanoramaDLL.
        'res = PP_Divide(datDst0(0), datDst1(0), datDst2(0), datSrc(0), parameter)
        res = PanoEnv.PP_Divide(datDst0, datDst1, datDst2, datSrc, parameter)   'x64

        'Store bitmap data
        bmpaccSrc.StoreRawData(bmpSrc)

        If (parameter.numOfDiv > 0) Then
            bmpaccDst0.StoreRawData(bmpDiv(0))
        End If

        If (parameter.numOfDiv > 1) Then
            bmpaccDst1.StoreRawData(bmpDiv(1))
        End If

        If (parameter.numOfDiv > 2) Then
            bmpaccDst2.StoreRawData(bmpDiv(2))
        End If

        Return res

    End Function

	'*! -----------------------------------------------------------------------------
	'*	@brief	: Process the 1st position of bitmap data to the panorama image.
	'*	@param	:
	'*	@return	: >= 0 : Success	< 0: Failure
	'* ------------------------------------------------------------------------------
	Public Function Pano_ProcImage1st(ByRef bmp As Bitmap, ByRef parameter As StructPanoramaImageParam, ByRef iccFilename As String, ByRef lutFilename As String) As Integer
		Dim res As Integer
		Dim bmpaccWork As BitmapAccess = New BitmapAccess
		Dim datWork() As Byte

		' Processing the right side image.
		datWork = bmpaccWork.LoadRawData(bmp)
        res = PanoEnv.PP_ProcessLhs(datWork, iccFilename, lutFilename, parameter.overlapWidth, parameter.srcWidth, parameter.srcHeight)
		bmpaccWork.StoreRawData(bmp)

		Return res
	End Function

	'*! -----------------------------------------------------------------------------
	'*	@brief	: Process the middle position of bitmap data to the panorama image.
	'*	@param	:
	'*	@return	: >= 0 : Success	< 0: Failure
	'* ------------------------------------------------------------------------------
	Public Function Pano_ProcImageMid(ByRef bmp As Bitmap, ByRef parameter As StructPanoramaImageParam, ByRef iccFilename As String, ByRef lutFilename As String) As Integer
		Dim res As Integer
		Dim bmpaccWork As BitmapAccess = New BitmapAccess
		Dim datWork() As Byte

		' Processing the right side image.
		datWork = bmpaccWork.LoadRawData(bmp)
        res = PanoEnv.PP_ProcessBoth(datWork, iccFilename, lutFilename, parameter.overlapWidth, parameter.srcWidth, parameter.srcHeight)
		bmpaccWork.StoreRawData(bmp)

		Return res
	End Function

	'*! -----------------------------------------------------------------------------
	'*	@brief	: Process the last position of bitmap data to the panorama image.
	'*	@param	:
	'*	@return	: >= 0 : Success	< 0: Failure
	'* ------------------------------------------------------------------------------
	Public Function Pano_ProcImageLast(ByRef bmp As Bitmap, ByRef parameter As StructPanoramaImageParam, ByRef iccFilename As String, ByRef lutFilename As String) As Integer
		Dim res As Integer
		Dim bmpaccWork As BitmapAccess = New BitmapAccess
		Dim datWork() As Byte

		' Processing the right side image.
		datWork = bmpaccWork.LoadRawData(bmp)
        res = PanoEnv.PP_ProcessRhs(datWork, iccFilename, lutFilename, parameter.overlapWidth, parameter.srcWidth, parameter.srcHeight)
		bmpaccWork.StoreRawData(bmp)

		Return res
	End Function

	'*! -----------------------------------------------------------------------------
	'*	@brief	: Set correction parameter.
	'*	@param	:
	'*	@return	: >= 0 : Success	< 0: Failure
	'* ------------------------------------------------------------------------------
	Public Function Pano_SetCorrectionParam(ByVal temp As Double, ByVal humid As Double, ByVal tblFilename As String) As Integer
        Return PanoEnv.PP_SetCorrectionParam(temp, humid, tblFilename)
	End Function

End Module

