﻿Imports System.Runtime.InteropServices
Imports System.Drawing.Printing

Public Class PrinterControler

	Private Const BUFF_SIZE As Integer = 256

	Private sPrintDoc As PrintDocument
	Private sTimer As Timer
	Private sbConnect As Boolean = False
	Private sPrintImageArray As ArrayList
	Private sLifeCountAtStart As Integer
	Private sLifeCountAtFin As Integer


	Private Const NUM_OF_IMAGES As Integer = 3	'Number of image files

    Public sHasMorePage As Integer              'Multi cut
    Public sPrintCount As Integer               'Printing counter
    Public sPrnSet As StructPrintSetting        'Structure of print setting
	Private sPaperSize As Integer
	Private sbAbort As Boolean = False

	Private sCfgResolution As Integer
	Private sCfgIsMatte As Boolean



	Sub New()
		' Declare the print object.
		sPrintDoc = New PrintDocument
		sPrintImageArray = New ArrayList

		' Register the handler functions of the printing process.
		AddHandler sPrintDoc.PrintPage, AddressOf pd_PrintPage
		AddHandler sPrintDoc.QueryPageSettings, AddressOf pd_QueryPageSettings
	End Sub

	'*! -----------------------------------------------------------------------------
	'*	@section	: Setter.
	'* ------------------------------------------------------------------------------
    Public Sub SetPprSize(ByVal size As Integer)
        sPrnSet.PaperSize = PaperSize(size).Number
    End Sub

    Public Sub SetResolution(ByVal reso As Integer)
        If reso <> 300 And reso <> 600 Then
            Return
        End If
        sPrnSet.PrintQuality = reso
        sPrnSet.YResolution = reso
    End Sub

    Public Sub SetHighDensityMode(ByVal mode As Integer)
        If mode <> 0 And mode <> 1 Then
            Return
        End If
        sPrnSet.HighDensityMode = mode
    End Sub

    Public Sub SetOvercoat(ByVal overcoat As Integer)
        sPrnSet.OvercoatFinish = overcoat
    End Sub

	Public Function SetPanoramaSetting(ByVal bLast As Boolean, ByVal overlapInch As Single) As Boolean
        Dim continueVal As Integer

        If bLast = True Then
            continueVal = 0
        Else
            continueVal = 1
        End If
        Return PRINTER_CTRL_ContPanorama(continueVal, CInt(overlapInch * 100.0))

	End Function
	Public Function AddPrintImage(ByRef bmpImage As Bitmap) As Integer

		sPrintImageArray.Add(bmpImage.Clone)

		Return sPrintImageArray.Count()
	End Function

	'*! -----------------------------------------------------------------------------
	'*	@section	: Getter.
	'* ------------------------------------------------------------------------------
	Public Function IsConnected() As Boolean
		Return sbConnect
	End Function

	Public Function GetFirmwareVer() As String
		Return PRINTER_INFO_FirmwareVersion()
	End Function

	Public Function GetImageCount() As Integer
		Return sPrintImageArray.Count()
	End Function

    Public Function CheckEnablePanorama() As Boolean
        Dim rdata As String = ""
        Return PRINTER_INFO_PanoramaPrint(rdata)
    End Function

	'Get the number of media remaining.
	Public Function GetMediaRemainCount() As Integer
        'Return (PRINTER_INFO_MediaRemainCount() - 50)
        Return PRINTER_INFO_MediaRemainCount()
    End Function

	Public Function GetImageCountInTable() As Integer
		Return sPrintImageArray.Count
	End Function

	Public Function GetStatusString() As String
		If Not sbConnect Then
			' Not connected
			Return "N/A"
		End If

		Return GetStateStr(sPortNo)
	End Function

    Public Function GetMediaClassString() As String
        If Not sbConnect Then
            ' Not connected
            Return "N/A"
        End If

        Return GetMediaClassStr(sPortNo)
    End Function


	' Get sensor infomation.
	Public Function GetSensorValue(ByRef mediaTemp As Single, ByRef humid As Single) As Integer
		Dim headTemp As Single
		Dim headVolt As Single

		Return CX2_GetTempSensorInfo(headTemp, mediaTemp, headVolt, humid)
	End Function


	'*! -----------------------------------------------------------------------------
	'*	@section	: Method
	'* ------------------------------------------------------------------------------
	Public Function CheckFreeBuffer() As Integer
        Return PrgEnv.GetFreeBuffer(sPortNo)
	End Function

	Public Function DoPrint(Optional ByRef bmpImage As Bitmap = Nothing, Optional ByVal bBlockingCall As Boolean = False) As Boolean
		Dim printerStatus As Integer
		Dim bRet As Boolean


		If bmpImage IsNot Nothing Then
			AddPrintImage(bmpImage)
		End If

		If sPrintImageArray.Count() <= 0 Then
			' Image data empty.
			Return False
		End If

		' Print setting.
		SetPrintConfig()

		' Check printable.
        printerStatus = PrgEnv.GetStatus(sPortNo)
        If printerStatus <> STATUS_USUALLY_IDLE And _
           printerStatus <> STATUS_USUALLY_COOLING And _
           printerStatus <> STATUS_USUALLY_MOTCOOLING And _
           printerStatus <> STATUS_USUALLY_STANDBY_MODE And _
           printerStatus <> STATUS_USUALLY_PRINTING Then
            GoTo ExitProocess
        End If

        ' Flush events in que.
        System.Windows.Forms.Application.DoEvents()

        ' Do print
        bRet = PrintImage(bBlockingCall)

ExitProocess:
        FileClose()
        Return bRet
	End Function

	Private Sub SetPrintConfig()
        ' Set print settings.
        sPrintDoc.PrinterSettings.PrinterName = sPrinterName
        PrintSetting(sPrnSet, sPrintDoc)

        ' Set the number of print job.
        sHasMorePage = PaperSize(sPaperSize).MultiCut

        ' Do not show "printing" dialog. 
        sPrintDoc.PrintController = New System.Drawing.Printing.StandardPrintController

        Return
    End Sub

	Private Sub setDefaultSetting()

        If PAPER_A4 <> -1 Then sPaperSize = PAPER_A4
        If PAPER_8x12 <> -1 Then sPaperSize = PAPER_8x12
        If PAPER_8x10 <> -1 Then sPaperSize = PAPER_8x10
        If PAPER_A5 <> -1 Then sPaperSize = PAPER_A5

		'-------------------------------------------
		'Get user setting
		'-------------------------------------------
		'Paper Size
		sPrnSet.PaperSize = PaperSize(sPaperSize).Number

		'Orientation
		sPrnSet.Orientation = DM_ORIENT_LANDSCAPE

		'Print Quality
		sPrnSet.PrintQuality = 300
		sPrnSet.YResolution = 300

        'High Denisy Mode
        sPrnSet.HighDensityMode = 0

		'Border
		sPrnSet.Border = DM_BORDER_OFF

		'Sharpness
		sPrnSet.Sharpness = 2

		'ICM
		sPrnSet.ICMMethod = DM_ICMMETHOD_NONE

		'Color Adjustment
		sPrnSet.ColorAdjustment = DM_EXCOLORADJ_NONE
		'Gamma
		sPrnSet.AdjGammaR = 0
		sPrnSet.AdjGammaG = 0
		sPrnSet.AdjGammaG = 0
		'Brightness
		sPrnSet.AdjBrightnessR = 0
		sPrnSet.AdjBrightnessG = 0
		sPrnSet.AdjBrightnessB = 0
		'Contrast
		sPrnSet.AdjContrastR = 0
		sPrnSet.AdjContrastG = 0
		sPrnSet.AdjContrastB = 0
		'Chroma
		sPrnSet.AdjChromaR = 0
		sPrnSet.AdjChromaG = 0
		sPrnSet.AdjChromaB = 0

		'Overcoat Finish (Glossy/Matte）
        sPrnSet.OvercoatFinish = DM_OVERCOAT_GLOSSY

		'Print Re-try
		sPrnSet.PrintRetry = DM_PRN_RETRY_OFF

		'2inch Cut
		sPrnSet.Cut2inch = DM_CUT2INCH_OFF

	End Sub

	Public Function OpenPort(ByRef printerName As String, ByRef portName As String) As Boolean

		sPrinterName = printerName
		sPortName = portName

		If sPrinterName <> "None" Then
			Try
				'Initialize & Get the port number.
                sPortNo = PrgEnv.CvInitialize(sPortName)

				If sPortNo < 0 Then
					' Failure.
					GoTo _EndFunction
				End If

                If PrgEnv.RegisterPrinterName(sPortNo, sPrinterName) <= 0 Then
                    ' Failure.
                    GoTo _EndFunction
                End If

				' Prepare printer settings.
				sDevPrinterName = sPrinterName
				GetPaperSizeInfo()
				ExDevModeTopSearch()

				' Set default setting.
				setDefaultSetting()

			Catch ex As Exception

				MsgBox(ex.ToString)
				GoTo _EndFunction
			End Try
		End If

		' Update the connect status to "Has connected printer".
		sbConnect = True

		Return True
_EndFunction:

		' Update the connect status to "No connected printer".
		sbConnect = False
		Return False
	End Function

	'*! -----------------------------------------------------------------------------
	'*	@section	: Private function.
	'* ------------------------------------------------------------------------------
	Private Function PrintImage(Optional ByVal bBlockingCall As Boolean = False) As Boolean
		Dim bRet As Boolean = True
		Dim printerStatus As Integer


		sbAbort = False

		If sPrintImageArray.Count() = 0 Then
			' Data empty.
			Return False
		End If

		'If enable the print end wait, then set a life counter at the start of printing.
		If bBlockingCall Then
			'Set a life counter at the start of printing
            sLifeCountAtStart = PrgEnv.GetCounterL(sPortNo)
			sLifeCountAtFin = sLifeCountAtStart + sPrintImageArray.Count()
		End If

		Do
			' Abort?
			If sbAbort = True Then
				GoTo _ExitProocess
			End If

			' confirm the free buffers in the printer.
            If PrgEnv.GetFreeBuffer(sPortNo) > 0 Then

                ' Execute print.
                sPrintDoc.Print()

                ' Complete transmission of the print data?
                If sPrintImageArray.Count() = 0 Then
                    Exit Do
                End If
            End If

			' Flush events in que.
			System.Windows.Forms.Application.DoEvents()
            Sleep(1000)
		Loop

		If Not bBlockingCall Then
			Return Not sbAbort
		End If

		Do	' Wait to complete print.
			If sbAbort = True Then
				GoTo _ExitProocess
			End If

			' Flush events in que.
			System.Windows.Forms.Application.DoEvents()

			' Confirm complete.
            If PrgEnv.GetCounterL(sPortNo) >= sLifeCountAtFin Then
                Exit Do
            End If

			' Check printer status
            printerStatus = PrgEnv.GetStatus(sPortNo)

            If printerStatus <> STATUS_USUALLY_IDLE And _
              printerStatus <> STATUS_USUALLY_COOLING And _
              printerStatus <> STATUS_USUALLY_MOTCOOLING And _
              printerStatus <> STATUS_USUALLY_STANDBY_MODE And _
              printerStatus <> STATUS_USUALLY_PRINTING Then
                MsgBox("Printing error. Abort printing.", MsgBoxStyle.OkOnly Or MsgBoxStyle.SystemModal)
                sbAbort = True
                GoTo _ExitProocess
            End If

            Sleep(1000)
        Loop

_ExitProocess:
		Return bRet
	End Function

	'*! -----------------------------------------------------------------------------
	'*	@section	: Event handler.
	'* ------------------------------------------------------------------------------
	Private Sub pd_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs)

		If sPrintImageArray.Count() > 0 Then
			e.Graphics.DrawImage(sPrintImageArray(0), 0, 0, e.PageBounds.Width, e.PageBounds.Height)
		End If

		sPrintImageArray.Remove(sPrintImageArray(0))

		sHasMorePage -= 1
		sPrintCount += 1

		If sHasMorePage > 0 Then
			e.HasMorePages = True
		Else
			e.HasMorePages = False
		End If

	End Sub

	Protected Sub pd_QueryPageSettings(ByVal sender As Object, ByVal e As QueryPageSettingsEventArgs)
		System.Windows.Forms.Application.DoEvents()
	End Sub

End Class
