/*
** PanoDef.h
*/ 

#include <windows.h>
#include <stdio.h>
#include <winioctl.h>

#ifndef _PANODEF_H_
#define _PANODEF_H_


/*! -----------------------------------------------------------------------------
 *	@section	: Result code definitions.
 * -----------------------------------------------------------------------------*/
#define NORMAL							(0)			// Successful
#define ERR_FILE_OPEN					(-1)		// Failed to file open.
#define ERR_NO_MEMORY					(-2)		// Free memory is empty.
#define	ERR_PARAM						(-3)		// Invalid parameter.
#define	ERR_OBJ							(-4)		// State is invalid.
#define ERR_NO_DATA						(-5)		// No data.
#define ERR_INTERNAL_PARAM				(-6)		// Invalid internal parameter.
#define	ERR_INTERNAL_PROC				(-7)		// Internal error.
#define ERR_NO_SUPPORT					(-8)		// Unsupported.
#define	ERR_INTERNAL_NO_DUTY			(-9)		// No duty data.
#define ERR_INVALID_ENVIRONMENT			(-10)		// Invalid Environment.

#define	ERR_ICCTBLFILE_OPEN				(-101)		// Failed to ICC file open.
#define	ERR_ICCTBLFILE_NODATA			(-105)		// Failed to ICC file open.
#define	ERR_ICCTBLFILE_FORMAT			(-110)		// Failed to ICC file open.
#define	ERR_ICCPROFILE_OPEN				(-201)		// Failed to ICC file open.
#define	ERR_ICCPROFILE_INIT				(-206)		// Error in Create convert object of ICM.
#define	ERR_ICCPROFILE_PROC				(-207)		// ICM conversion failure.

#define	ERR_LUTFILE_OPEN				(-300)		// Failed to Look-up table file open.
#define	ERR_LUTFILE_NODATA				(-305)		// No data.
#define ERR_LUTFILE_FORMAT				(-310)		// File format error of LUT file.
#define ERR_LUTFILE_ELM_TOOSMALL		(-311)		// Element number of LUT file is too small.


#define	ERR_CORRFILE_OPEN				(-400)		// Failed to Correction table file open.
#define ERR_CORRFILE_NODATA				(-405)		// No data.
#define ERR_CORRFILE_FORMAT				(-410)		// File format error of the Correction table file.
#define ERR_CORRFILE_DUPLICATE			(-411)		// Duplicate data.

/*! -----------------------------------------------------------------------------
 *	@section	: Structure definitions.
 * -----------------------------------------------------------------------------*/
// The divide image parameters.
typedef struct _StructDivideParam
{
	unsigned long	numOfDiv;		// Number of divided.
	unsigned long	overlapWidth;	// Overlap width. [dot]
	unsigned long	srcWidth;		// Width of source image data. [dot]
	unsigned long	srcHeight;		// Height of source image data. [dot]
	unsigned long	dstWidth;		// Width of the divided image data.	[dot]
	unsigned long	dstHeight;		// Height of the divided image data.	[dot]
}	StructDivideParam;

// The process image parameters.
typedef struct _StrutProcessParam
{
	unsigned long	overlapWidth;	// Overlap width. [dot]
	unsigned long	srcWidth;		// Width of source image data. [dot]
	unsigned long	srcHeight;		// Height of source image data. [dot]
}	StrutProcessParam;

// The duty image parameters.
typedef struct _StructImgDutyParam
{
	unsigned long	srcWidth;		// Width of source image data. [dot]
	unsigned long	srcHeight;		// Height of sosurce image data. [dot]
	double	dutyY;					// RHS duty Y defined by LUT file.
	double	dutyM;					// RHS duty Y defined by LUT file.
	double	dutyC;					// RHS duty Y defined by LUT file.
}	StructImgDutyParam;

// The duty infomation of Look-up table.
typedef struct _StructDutyOfLUTFile
{
	unsigned long	pos;			// Location in the Look-up table to get the Duty.
	unsigned long	scale;			// alignment dummy.
	double resRhsDutyY;				// RHS duty Y defined by LUT file. (Output)
	double resRhsDutyM;				// RHS duty M defined by LUT file. (Output)
	double resRhsDutyC;				// RHS duty C defined by LUT file. (Output)
	double resLhsDutyY;				// LHS duty Y defined by LUT file. (Output)
	double resLhsDutyM;				// LHS duty M defined by LUT file. (Output)
	double resLhsDutyC;				// LHS duty C defined by LUT file. (Output)
}	StructDutyOfLUTFile;


/*! -----------------------------------------------------------------------------
 *	@section	: Declare DLL Functions.
 * -----------------------------------------------------------------------------*/
#ifdef __cplusplus
extern "C" {
#endif

/*! ------------------------
 *	Process panorama image.
 *  ------------------------*/
extern long APIENTRY PP_Reset( void );
extern long APIENTRY PP_Divide( char *pImgDst0, char *pImgDst1, char *pImgDst2, char *pImgSrc, StructDivideParam *pParam );
extern long APIENTRY PP_ProcessLhs( char *pImgData, WCHAR *iccTblFilename, WCHAR *lutFilename, unsigned long overlapWidth, unsigned long srcWidth, unsigned long srcHeight );
extern long APIENTRY PP_ProcessRhs( char *pImgData, WCHAR *iccTblFilename, WCHAR *lutFilename, unsigned long overlapWidth, unsigned long srcWidth, unsigned long srcHeight );
extern long APIENTRY PP_ProcessBoth( char *pImgData, WCHAR *iccTblFilename, WCHAR *lutFilename, unsigned long overlapWidth, unsigned long srcWidth, unsigned long srcHeight );
extern long APIENTRY PP_SetCorrectionParam( double temp, double humidity, WCHAR *envTblFilename );

#ifdef __cplusplus
}
#endif


#endif
// End of PanoDef.h
