Option Strict Off
Option Explicit On
Imports VB = Microsoft.VisualBasic
Imports System.Runtime.InteropServices

Friend Class Form1
	Inherits System.Windows.Forms.Form

	Const ESC As Short = &H1BS

	Private PortName As String					'USB port name
	Private PortNo As Integer					'Port number
	Private CurrentImageNum As Integer			'Printing image number

	Public PrnSet As PRINT_SETTING
	Public PAPER_SIZE_8x12 As Integer


	'==================================================
	' Form Load
	'==================================================
	Private Sub Form1_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load

		'-----------------------------------------
		'File Path / File Name / Printer Name
		'-----------------------------------------
		PrinterName = "DP-DS820"		'Printer Name

		'-----------------------------------------
		'Get port name
		'-----------------------------------------
		PortName = GetPortName(PrinterName)
		If PortName = "" Then
			MsgBox("Printer(" & PrinterName & ") is not connected. ", MsgBoxStyle.OkOnly)
			End
		End If
		PortNo = PortInitialize(PortName) 'Initialize of Status API

		PrintSettingIni()

	End Sub

	'---------------------------------------------------------
	'Print Setting Ini
	'---------------------------------------------------------
	Public Sub PrintSettingIni()

		'Get index of (8x12) paper size.
		PAPER_SIZE_8x12 = getPaperID("(8x12)")

		ExDevModeTopSearch()
		SetPrintSettingInfo()

	End Sub


	'---------------------------------------------------------
	'Devmode Ini
	'---------------------------------------------------------
	Public Sub SetPrintSettingInfo()
		PrnSet.PaperSize = PAPER_SIZE_8x12
		PrnSet.Orientation = DM_ORIENT_LANDSCAPE
		PrnSet.PrintQuality = 300
		PrnSet.YResolution = 300
		PrnSet.ICMMethod = DM_ICMMETHOD_SYSTEM
		PrnSet.Border = DM_BORDER_OFF
		PrnSet.Sharpness = 2
		PrnSet.ColorAdjustment = DM_EXCOLORADJ_NONE
		PrnSet.AdjGammaR = 0
		PrnSet.AdjGammaG = 0
		PrnSet.AdjGammaB = 0
		PrnSet.AdjBrightnessR = 0
		PrnSet.AdjBrightnessG = 0
		PrnSet.AdjBrightnessB = 0
		PrnSet.AdjContrastR = 0
		PrnSet.AdjContrastG = 0
		PrnSet.AdjContrastB = 0
		PrnSet.AdjChromaR = 0
		PrnSet.AdjChromaG = 0
		PrnSet.AdjChromaB = 0
		PrnSet.OvercoatFinish = DM_OVERCOAT_GLOSSY
		PrnSet.PrintRetry = DM_PRN_RETRY_OFF
		PrnSet.Cut2inch = DM_CUT2INCH_OFF
		PrnSet.HighDensity = DM_HIGHDENSITY_OFF
	End Sub

	'==================================================
	'Print
	'==================================================
	Private Sub cmdPrint1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPrint1.Click
		Dim counterAtFin As Integer
		Dim printerStatus As Integer
		Dim cutterMode As Integer

		'Check exit image files.
		If System.IO.File.Exists(textFilename1.Text) = False Then
			MsgBox("Not exist the image file 1.")
			Return
		End If
		If System.IO.File.Exists(textFilename2.Text) = False Then
			MsgBox("Not exist the image file 2.")
			Return
		End If

		If CInt(numOfImage.Value) > 2 Then
			If System.IO.File.Exists(textFilename3.Text) = False Then
				MsgBox("Not exist the image file 3.")
				Return
			End If
		End If

		pd.PrinterSettings.PrinterName = PrinterName
		PrnSet.PaperSize = PAPER_SIZE_8x12
		PrintSetting(PrnSet, pd)
		CurrentImageNum = 0

		GroupBox_Panorama.Enabled = False

		counterAtFin = GetCounterL(PortNo) + CInt(numOfImage.Value)

		'Check printable panorama
		If GetPanoramaPrintable(PortNo) <> STATUS_PRINTABLE Then
			MsgBox("Now state is unprintable panorama.", MsgBoxStyle.OkOnly Or MsgBoxStyle.SystemModal)
			GroupBox_Panorama.Enabled = True
			Exit Sub
		End If

		'Check request all of the data.
		Do
			' confirm the free buffers in the printer.
			If GetFreeBuffer(PortNo) > 0 Then

				If (CurrentImageNum + 1) < CInt(numOfImage.Value) Then
					cutterMode = CUTTER_MODE_PANORAMA
				Else
					cutterMode = CUTTER_MODE_STANDARD
				End If

				' Send the cutter control command.
				If SetCutterMode(PortNo, cutterMode) = False Then
					MsgBox("Printing error.", MsgBoxStyle.OkOnly Or MsgBoxStyle.SystemModal)
					GroupBox_Panorama.Enabled = True
					Exit Sub
				End If

				' Execute print.
				Try
					pd.Print()
				Catch ex As Exception
					' error message
					MessageBox.Show(ex.Message)
				End Try
			End If

			'Check 
			If CurrentImageNum >= CInt(numOfImage.Value) Then
				Exit Do
			End If

			' Flush events.
			System.Windows.Forms.Application.DoEvents()
			System.Threading.Thread.Sleep(100)
		Loop

		GroupBox_Panorama.Enabled = True

		'Print Wait for end. 
		Do
			If GetCounterL(PortNo) >= counterAtFin Then
				Exit Do
			End If

			' Check printer status
			printerStatus = GetStatus(PortNo)

			If printerStatus <> STATUS_USUALLY_IDLE And _
			  printerStatus <> STATUS_USUALLY_MOTCOOLING And _
			  printerStatus <> STATUS_USUALLY_STANDBY_MODE And _
			  printerStatus <> STATUS_USUALLY_PRINTING Then
				MsgBox("Printing error.", MsgBoxStyle.OkOnly Or MsgBoxStyle.SystemModal)
				Exit Do
			End If

			' Flush events.
			System.Windows.Forms.Application.DoEvents()
			System.Threading.Thread.Sleep(100)

		Loop
	End Sub

	'==================================================
	'PrintPage Handler
	'==================================================
	Private Sub pd_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles pd.PrintPage

		Dim img As Image
		Dim Ratio As Single
		Dim Print_Width As Single
		Dim Print_Height As Single

		' Switch image data
		Select Case CurrentImageNum
			Case 0
				img = System.Drawing.Image.FromFile(textFilename1.Text)
			Case 1
				img = System.Drawing.Image.FromFile(textFilename2.Text)
			Case Else
				img = System.Drawing.Image.FromFile(textFilename3.Text)
		End Select

		Ratio = e.PageBounds.Height / img.Height
		Print_Width = img.Width * Ratio
		Print_Height = img.Height * Ratio


		If Print_Width < e.PageBounds.Width Then
			Ratio = e.PageBounds.Width / img.Width
			Print_Width = img.Width * Ratio
			Print_Height = img.Height * Ratio
		End If

		'Centering
		Dim OverWidth As Single = e.PageBounds.Width - Print_Width
		Dim OverHeight As Single = e.PageBounds.Height - Print_Height
		Dim imgTop As Single = OverWidth / 2
		Dim imgLeft As Single = OverHeight / 2

		e.Graphics.DrawImage(img, imgTop, imgLeft, Print_Width, Print_Height)
		e.HasMorePages = False

		CurrentImageNum += 1

		img.Dispose()

	End Sub

	Private Sub numOfImage_ValueChanged(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles numOfImage.ValueChanged
		If CInt(numOfImage.Value) = 2 Then
			textFilename3.Enabled = False
		Else
			textFilename3.Enabled = True
		End If
	End Sub
End Class
