Imports System.Runtime.InteropServices

Module Paper
	'Paper size information
	Structure StructPaperSize
		Dim Name As String		'size name
		Dim Number As Integer	'size number
	End Structure

	'paper size
	<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
	Structure SIZEL
		Dim x As Int32
		Dim y As Int32
	End Structure

	'image area
	<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
	Structure RECTL
		Dim left As Int32
		Dim top As Int32
		Dim right As Int32
		Dim bottom As Int32
	End Structure

	'paper information
	<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
	Structure FORM_INFO_1
		Dim Flags As UInt32
		Dim pName As IntPtr
		Dim Size As SIZEL			'paper size
		Dim ImageableArea As RECTL	'image area
	End Structure

	'printer access
	<StructLayout(LayoutKind.Sequential, CharSet:=CharSet.Auto)> _
	Structure PRINTER_DEFAULTS
		Dim pDataType As IntPtr
		Dim pDevMode As IntPtr
		Dim DesiredAccess As UInt32
	End Structure

	'Standard rights, Combines DELETE, READ_CONTROL, WRITE_DAC, and WRITE_OWNER access.
	Public Const STANDARD_RIGHTS_REQUIRED As UInt32 = &HF0000
	'Printing-specific access rights for printers to perform administrative tasks.
	Public Const PRINTER_ACCESS_ADMINISTER As UInt32 = &H4&
	'Printing-specific access rights for printers to perform basic printing operations.
	Public Const PRINTER_ACCESS_USE As UInt32 = &H8&
	'Access rights for printers to perform all administrative tasks and basic printing operations except synchronization.
	Public Const PRINTER_ALL_ACCESS As UInt32 = _
	   (STANDARD_RIGHTS_REQUIRED Or _
	 PRINTER_ACCESS_ADMINISTER Or _
	 PRINTER_ACCESS_USE)
	Public PaperSize(MediaNumMax - 1) As StructPaperSize

	Declare Function DeviceCapabilities Lib "winspool.drv" Alias "DeviceCapabilitiesW" ( _
		ByVal pDevice As String, _
		ByVal pPort As String, _
		ByVal fwCapability As Integer, _
		ByVal pOutput As IntPtr, _
		ByVal pDevMode As IntPtr) As Integer

	Declare Function EnumForms Lib "winspool.drv" Alias "EnumFormsW" ( _
		ByVal hPrinter As Integer, _
		ByVal Level As Integer, _
		ByVal pForm As IntPtr, _
		ByVal cbBuf As Integer, _
		ByRef pcbNeeded As Integer, _
		ByRef pcReturned As Integer) As <MarshalAs(UnmanagedType.Bool)> Boolean


	Const DC_PAPERNAMES As Int32 = 16
	Const DC_PAPERS As Int32 = 2


	'----------------------------------------------------------
	' Get paper ID
	'----------------------------------------------------------
	Public Function getPaperID(ByVal paperName As String) As Integer
		Dim hPrinter As IntPtr
		Dim bResultCode As Boolean
		Dim sPrinterDefaults As PRINTER_DEFAULTS
		Dim lpPrinterDefaults As IntPtr
		Dim formInfoSize As Integer
		Dim formInfoReturned As Integer
		Dim bytformInfoData() As Byte
		Dim formInfoArray() As FORM_INFO_1 = Nothing
		Dim hFormInfo As Integer
		Dim lpFormInfo As IntPtr
		Dim i As Integer
		Dim strTemp As String = New String("", 255)
		Dim paperID As Integer

		With sPrinterDefaults
			.pDataType = IntPtr.Zero
			.pDevMode = IntPtr.Zero
			.DesiredAccess = PRINTER_ALL_ACCESS
		End With

		lpPrinterDefaults = Marshal.AllocHGlobal(Marshal.SizeOf(sPrinterDefaults))

		Try

			Marshal.StructureToPtr(sPrinterDefaults, lpPrinterDefaults, False)

			bResultCode = OpenPrinter(PrinterName, hPrinter, lpPrinterDefaults)

			bResultCode = EnumForms(hPrinter, 1, IntPtr.Zero, 0, formInfoSize, formInfoReturned)
			If formInfoSize = 0 Then
				ClosePrinter(hPrinter)
				Exit Function
			End If

			hFormInfo = GlobalAlloc(GHND, formInfoSize)
			lpFormInfo = GlobalLock(hFormInfo)
			bResultCode = EnumForms(hPrinter, 1, lpFormInfo, formInfoSize, formInfoSize, formInfoReturned)
			If bResultCode = False Then
				GlobalFree(lpFormInfo)
				ClosePrinter(hPrinter)
				Exit Function
			End If

			ReDim bytformInfoData(formInfoSize - 1)

			Marshal.Copy(lpFormInfo, bytformInfoData, 0, formInfoSize)

			ReDim formInfoArray(formInfoReturned - 1)

			formInfoArray = byte2FormInfoArray(bytformInfoData, formInfoReturned)

			paperID = 0
			For i = 0 To formInfoReturned - 1
				strTemp = Marshal.PtrToStringAuto(formInfoArray(i).pName)
				If strTemp = paperName Then
					paperID = i + 1
					Exit For
				End If
			Next

		Catch ex As Exception

		Finally
			Marshal.FreeHGlobal(lpPrinterDefaults)
		End Try

		GlobalFree(lpFormInfo)
		lpFormInfo = Nothing
		ClosePrinter(hPrinter)

		Return paperID

	End Function

	'----------------------------------------------------------
	' Convert from byte array to form information
	'----------------------------------------------------------
	Private Function byte2FormInfoArray(ByVal byteArray() As Byte, ByVal arraySize As Integer) As FORM_INFO_1()
		Dim addrOffset As Integer
		Dim i As Integer
		Dim formInfoArray() As FORM_INFO_1
		Dim f64bitOS As Boolean
		ReDim formInfoArray(arraySize - 1)

		f64bitOS = Is64bitOS()

		addrOffset = 0
		For i = 0 To arraySize - 1
			'Flag
			formInfoArray(i).Flags = BitConverter.ToUInt32(byteArray, addrOffset)
			addrOffset += 4

			If f64bitOS Then
				'64bit OS
				'padding for 64bit OS
				addrOffset += 4

				'pName
				formInfoArray(i).pName = CType(BitConverter.ToUInt64(byteArray, addrOffset), IntPtr)
				addrOffset += 8
			Else
				'32bit OS
				'pName
				formInfoArray(i).pName = CType(BitConverter.ToUInt32(byteArray, addrOffset), IntPtr)
				addrOffset += 4
			End If

			'SIZE
			formInfoArray(i).Size.x = BitConverter.ToInt32(byteArray, addrOffset)
			addrOffset += 4

			formInfoArray(i).Size.y = BitConverter.ToInt32(byteArray, addrOffset)
			addrOffset += 4

			'RECT
			formInfoArray(i).ImageableArea.left = BitConverter.ToInt32(byteArray, addrOffset)
			addrOffset += 4

			formInfoArray(i).ImageableArea.top = BitConverter.ToInt32(byteArray, addrOffset)
			addrOffset += 4

			formInfoArray(i).ImageableArea.right = BitConverter.ToInt32(byteArray, addrOffset)
			addrOffset += 4

			formInfoArray(i).ImageableArea.bottom = BitConverter.ToInt32(byteArray, addrOffset)
			addrOffset += 4
		Next

		Return formInfoArray

	End Function

	'----------------------------------------------------------
	' check OS bitwidth
	'----------------------------------------------------------
	Private Function Is64bitOS() As Boolean
		Dim procArch As String = System.Environment.GetEnvironmentVariable("PROCESSOR_ARCHITECTURE")

		If procArch Is Nothing OrElse procArch = "x86" Then
			Dim pa6432 As String = System.Environment.GetEnvironmentVariable("PROCESSOR_ARCHITEW6432")

			If pa6432 Is Nothing OrElse pa6432 = "x86" Then
				'32bit OS
				Is64bitOS = False
			Else
				'64bit OS
				Is64bitOS = True
			End If
		Else
			'64bit OS
			Is64bitOS = True
		End If
	End Function

End Module
