Option Strict Off
Option Explicit On
Imports VB = Microsoft.VisualBasic
Imports System.Runtime.InteropServices

Friend Class Form1
	Inherits System.Windows.Forms.Form

	Const ESC As Short = &H1BS

	Private PortName As String					'USB port name
	Private PortNo As Integer					'Port number
	Private PrintImage As Image

	Public PrnSet As PRINT_SETTING
	Public PAPER_SIZE_8x10 As Integer

	'-----------------------------------------
	'Setting value and text in ComboBox
	'-----------------------------------------
	Public Structure ComboItem
		Public Text As String
		Public Number As Integer
		Public Overrides Function ToString() As String
			Return Text
		End Function
		Public Sub New(ByVal S As String, ByVal I As Integer)
			Text = S
			Number = I
		End Sub
	End Structure


	'==================================================
	' Form Load
	'==================================================
	Private Sub Form1_Load(ByVal eventSender As System.Object, ByVal eventArgs As System.EventArgs) Handles MyBase.Load

		'-----------------------------------------
		'File Path / File Name / Printer Name
		'-----------------------------------------
		PrinterName = "DP-DS820"		'Printer Name

		'-----------------------------------------
		'Setting of OvercoatFinish combo box
		'-----------------------------------------
		With ComboBox_Type
			.Items.Clear()
			.Items.Add(New ComboItem("Matte", OVERCOAT_FINISH_PMATTE11))
			.Items.Add(New ComboItem("Fine Matte", OVERCOAT_FINISH_PMATTE12))
			.Items.Add(New ComboItem("Luster", OVERCOAT_FINISH_PMATTE13))
			.SelectedIndex = 0
		End With

		'-----------------------------------------
		'Get port name
		'-----------------------------------------
		PortName = GetPortName(PrinterName)
		If PortName = "" Then
			MsgBox("Printer(" & PrinterName & ") is not connected. ", MsgBoxStyle.OkOnly)
			End
		End If
		PortNo = PortInitialize(PortName) 'Initialize of Status API

		PrintSettingIni()

		'Initialize form item.
		ComboBox_Type.SelectedIndex = 0
	End Sub

	'---------------------------------------------------------
	'Print Setting Ini
	'---------------------------------------------------------
	Public Sub PrintSettingIni()
		Dim i As Integer

		GetPaperSizeInfo()

		'Get index of (8x10) paper size.
		For i = 0 To PaperSize.Length - 1
			If PaperSize(i).Name = "(8x10)" Then
				PAPER_SIZE_8x10 = PaperSize(i).Number
				Exit For
			End If
		Next

		ExDevModeTopSearch()
		SetPrintSettingInfo()

	End Sub


	'---------------------------------------------------------
	'Devmode Ini
	'---------------------------------------------------------
	Public Sub SetPrintSettingInfo()
		PrnSet.PaperSize = PAPER_SIZE_8x10
		PrnSet.Orientation = DM_ORIENT_LANDSCAPE
		PrnSet.PrintQuality = 300
		PrnSet.YResolution = 300
		PrnSet.ICMMethod = DM_ICMMETHOD_SYSTEM
		PrnSet.Border = DM_BORDER_OFF
		PrnSet.Sharpness = 2
		PrnSet.ColorAdjustment = DM_EXCOLORADJ_NONE
		PrnSet.AdjGammaR = 0
		PrnSet.AdjGammaG = 0
		PrnSet.AdjGammaB = 0
		PrnSet.AdjBrightnessR = 0
		PrnSet.AdjBrightnessG = 0
		PrnSet.AdjBrightnessB = 0
		PrnSet.AdjContrastR = 0
		PrnSet.AdjContrastG = 0
		PrnSet.AdjContrastB = 0
		PrnSet.AdjChromaR = 0
		PrnSet.AdjChromaG = 0
		PrnSet.AdjChromaB = 0
		PrnSet.OvercoatFinish = DM_OVERCOAT_GLOSSY
		PrnSet.PrintRetry = DM_PRN_RETRY_OFF
		PrnSet.Cut2inch = DM_CUT2INCH_OFF
		PrnSet.HighDensity = DM_HIGHDENSITY_OFF
	End Sub

	'==================================================
	'Print
	'==================================================
	Private Sub cmdPrint1_Click(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles cmdPrint1.Click
		Dim type As Integer
		Dim res As Boolean

		'Check exit image files.
		If System.IO.File.Exists(textFilenamePattern.Text) = False Then
			MsgBox("Not exist Matte pattern image.")
			Return
		End If
		If System.IO.File.Exists(textFilenameImage.Text) = False Then
			MsgBox("Not exist Print image.")
			Return
		End If

		GroupBox_PartialMatte.Enabled = False

		pd.PrinterSettings.PrinterName = PrinterName
		PrnSet.PaperSize = PAPER_SIZE_8x10
		PrintSetting(PrnSet, pd)

		'Set pattern image.
		type = DirectCast(ComboBox_Type.SelectedItem, ComboItem).Number

		res = SetOvercoatFinish(PortNo, type)

		If res <> True Then
			MsgBox("Error. (Overcoat setting)", MsgBoxStyle.OkOnly Or MsgBoxStyle.SystemModal)
			GroupBox_PartialMatte.Enabled = True
			Exit Sub
		End If

		'Read pattern file
		PrintImage = System.Drawing.Image.FromFile(textFilenamePattern.Text)

		'Partial matte pattern is transmitted 
		Try
			pd.Print()
		Catch ex As Exception
			' error message
			MessageBox.Show(ex.Message)
			GroupBox_PartialMatte.Enabled = True
			Exit Sub
		Finally
			If PrintImage IsNot Nothing Then
				PrintImage.Dispose()
			End If
		End Try

		'Read image file
		PrintImage = System.Drawing.Image.FromFile(textFilenameImage.Text)

		'Print image data is transmitted
		Try
			pd.Print()
		Catch ex As Exception
			' error message
			MessageBox.Show(ex.Message)
		Finally
			If PrintImage IsNot Nothing Then
				PrintImage.Dispose()
			End If
		End Try

		GroupBox_PartialMatte.Enabled = True

	End Sub


	'==================================================
	'PrintPage Handler
	'==================================================
	Private Sub pd_PrintPage(ByVal sender As Object, ByVal e As System.Drawing.Printing.PrintPageEventArgs) Handles pd.PrintPage
		Dim Ratio As Single
		Dim Print_Width As Single
		Dim Print_Height As Single

		Ratio = e.PageBounds.Height / PrintImage.Height
		Print_Width = PrintImage.Width * Ratio
		Print_Height = PrintImage.Height * Ratio


		If Print_Width < e.PageBounds.Width Then
			Ratio = e.PageBounds.Width / PrintImage.Width
			Print_Width = PrintImage.Width * Ratio
			Print_Height = PrintImage.Height * Ratio
		End If

		'Centering
		Dim OverWidth As Single = e.PageBounds.Width - Print_Width
		Dim OverHeight As Single = e.PageBounds.Height - Print_Height
		Dim imgTop As Single = OverWidth / 2
		Dim imgLeft As Single = OverHeight / 2

		e.Graphics.DrawImage(PrintImage, imgTop, imgLeft, Print_Width, Print_Height)

		e.HasMorePages = False
	End Sub
End Class