/*
** DS820Stat.h
*/ 


#ifndef _DS820STAT_H_
#define _DS820STAT_H_

/* Printer Status Code */
#define GROUP_USUALLY			(0x00010000)
#define GROUP_SETTING			(0x00020000)
#define GROUP_HARDWARE			(0x00040000)
#define GROUP_SYSTEM			(0x00080000)
#define GROUP_FLSHPROG			(0x00100000)

#define STATUS_ERROR			(0x80000000)

#define STATUS_USUALLY_IDLE				(GROUP_USUALLY | 0x0001)
#define STATUS_USUALLY_PRINTING			(GROUP_USUALLY | 0x0002)
#define STATUS_USUALLY_STANDSTILL		(GROUP_USUALLY | 0x0004)
#define STATUS_USUALLY_PAPER_END		(GROUP_USUALLY | 0x0008)
#define STATUS_USUALLY_RIBBON_END		(GROUP_USUALLY | 0x0010)
#define STATUS_USUALLY_COOLING			(GROUP_USUALLY | 0x0020)
#define STATUS_USUALLY_MOTCOOLING		(GROUP_USUALLY | 0x0040)
#define STATUS_USUALLY_STANDBY_MODE		(GROUP_USUALLY | 0x8000)

#define STATUS_SETTING_COVER_OPEN		(GROUP_SETTING | 0x0001)
#define STATUS_SETTING_PAPER_JAM		(GROUP_SETTING | 0x0002)
#define STATUS_SETTING_RIBBON_ERR		(GROUP_SETTING | 0x0004)
#define STATUS_SETTING_PAPER_ERR		(GROUP_SETTING | 0x0008)
#define STATUS_SETTING_DATA_ERR	        (GROUP_SETTING | 0x0010)
#define STATUS_SETTING_SCRAPBOX_ERR     (GROUP_SETTING | 0x0020)

#define STATUS_HARDWARE_ERR01			(GROUP_HARDWARE | 0x0001)
#define STATUS_HARDWARE_ERR02			(GROUP_HARDWARE | 0x0002)
#define STATUS_HARDWARE_ERR03			(GROUP_HARDWARE | 0x0004)
#define STATUS_HARDWARE_ERR04			(GROUP_HARDWARE | 0x0008)
#define STATUS_HARDWARE_ERR05			(GROUP_HARDWARE | 0x0010)
#define STATUS_HARDWARE_ERR06			(GROUP_HARDWARE | 0x0020)
#define STATUS_HARDWARE_ERR07			(GROUP_HARDWARE | 0x0040)
#define STATUS_HARDWARE_ERR08			(GROUP_HARDWARE | 0x0080)
#define STATUS_HARDWARE_ERR09			(GROUP_HARDWARE | 0x0100)
#define STATUS_HARDWARE_ERR10			(GROUP_HARDWARE | 0x0200)

#define STATUS_SYSTEM_ERR01				(GROUP_SYSTEM | 0x0001)

#define STATUS_FLSHPROG_IDLE			(GROUP_FLSHPROG + 0x0001)
#define STATUS_FLSHPROG_WRITING	        (GROUP_FLSHPROG + 0x0002)
#define STATUS_FLSHPROG_FINISHED		(GROUP_FLSHPROG + 0x0004)
#define STATUS_FLSHPROG_DATA_ERR1		(GROUP_FLSHPROG + 0x0008)
#define STATUS_FLSHPROG_DEVICE_ERR1     (GROUP_FLSHPROG + 0x0010)
#define STATUS_FLSHPROG_OTHERS_ERR1     (GROUP_FLSHPROG + 0x0020)

/* Cutter Mode */
#define CUTTER_MODE_STANDARD		(0)
#define CUTTER_MODE_NONSCRAP		(1)
#define CUTTER_MODE_2INCHCUT		(120)
#define CUTTER_MODE_PANORAMA		(1000)

/* Resolution Code for CWD */
#define CWD_300DPI			(0x00000000)
#define CWD_600DPI			(0x00000001)
#define CWD_610DPI			(0x00000002)
#define CWD_620DPI			(0x00000003)

/* Media Code for CWD */
#define CWD_RX1				(0x0001)
#define CWD_HQL				(0x0002)
#define CWD_HDM				(0x0003)

/* Status Code for PanoramaPrintable Command */
#define STATUS_PRINTABLE			(0x00000000)
#define STATUS_HIGHTEMP_HEAD		(0x00000001)
#define STATUS_HIGHTEMP_BOTH		(0x00000011)
#define STATUS_HIGHHUMIDITY			(0x00000100)
#define STATUS_LOWTEMP_MEDIA		(0x00000020)
#define STATUS_OTHER_STATE			(0x00010000)
#define STATUS_COMM_ERROR			(-1)

/* Overcoat Setting Value */
#define OVERCOAT_FINISH_GLOSSY		(0)
#define OVERCOAT_FINISH_MATTE1		(1)
#define OVERCOAT_FINISH_MATTE2		(2)
#define OVERCOAT_FINISH_MATTE3		(3)
#define OVERCOAT_FINISH_FINEMATTE	(21)
#define OVERCOAT_FINISH_LUSTER		(22)
#define OVERCOAT_FINISH_HSMATTE		(31)
#define OVERCOAT_FINISH_PMATTE11	(101)
#define OVERCOAT_FINISH_PMATTE12	(121)
#define OVERCOAT_FINISH_PMATTE13	(122)

/* Print Speed Control Define */
#define PRINTSPEED_NORMAL			(0x00000000)
#define PRINTSPEED_HIGH_DENSITY		(0x00000003)

#ifdef __cplusplus
extern "C" {
#endif

long APIENTRY PortInitialize( LPWSTR p );
long APIENTRY CvInitialize( LPWSTR p );

long APIENTRY CvGetVersion( long lPortNum, LPSTR p );
long APIENTRY CvGetSensorInfo( long lPortNum, LPSTR p );
long APIENTRY CvGetResolutionH( long lPortNum );
long APIENTRY CvGetResolutionV( long lPortNum );
long APIENTRY CvGetMedia( long lPortNum, LPSTR p );
long APIENTRY CvGetStatus( long lPortNum );
long APIENTRY CvGetCounterL( long lPortNum );
long APIENTRY CvGetCounterA( long lPortNum );
long APIENTRY CvGetCounterB( long lPortNum );
BOOL APIENTRY CvSetClearCounterA( long lPortNum );
BOOL APIENTRY CvSetClearCounterB( long lPortNum );
long APIENTRY CvGetFreeBuffer( long lPortNum );
long APIENTRY CvGetPQTY( long lPortNum );
long APIENTRY CvGetMediaCounter( long lPortNum );
long APIENTRY CvGetMediaColorOffset( long lPortNum );
long APIENTRY CvGetMediaLotNo( long lPortNum, LPSTR p );
long APIENTRY CvGetSerialNo( long lPortNum, LPSTR p );
BOOL APIENTRY CvSetFirmwUpdateMode( long lPortNum );
BOOL APIENTRY CvSetFirmwDataWrite( long lPortNum, LPSTR dBuff, DWORD bLen );
BOOL APIENTRY CvSetColorDataClear( long lPortNum );
BOOL APIENTRY CvSetColorDataWrite( long lPortNum, LPSTR dBuff, DWORD bLen );
BOOL APIENTRY CvSetColorDataVersion( long lPortNum, LPSTR dBuff, DWORD bLen );
long APIENTRY CvGetColorDataVersion( long lPortNum, LPSTR p );
long APIENTRY CvGetColorDataChecksum( long lPortNum, LPSTR p );
BOOL APIENTRY CvSetCommand( long lPortNum, LPSTR lpCmd, DWORD dwCmdLen );
long APIENTRY CvGetCommandEX( long lPortNum, LPSTR lpCmd, DWORD dwCmdLen, LPSTR lpRetBuff, DWORD dwRetBuffSize );

long APIENTRY GetFirmwVersion( long lPortNum, LPSTR p );
long APIENTRY GetSensorInfo( long lPortNum, LPSTR p );
long APIENTRY GetResolutionH( long lPortNum );
long APIENTRY GetResolutionV( long lPortNum );
long APIENTRY GetMedia( long lPortNum, LPSTR p );
long APIENTRY GetStatus( long lPortNum );
long APIENTRY GetCounterL( long lPortNum );
long APIENTRY GetCounterA( long lPortNum );
long APIENTRY GetCounterB( long lPortNum );
long APIENTRY GetCounterP( long lPortNum );
long APIENTRY GetCounterMatte( long lPortNum );
long APIENTRY GetCounterM( long lPortNum );
BOOL APIENTRY SetClearCounterA( long lPortNum );
BOOL APIENTRY SetClearCounterB( long lPortNum );
BOOL APIENTRY SetCounterP( long lPortNum, long lCounter );
BOOL APIENTRY SetClearCounterM( long lPortNum );
long APIENTRY GetFreeBuffer( long lPortNum );
long APIENTRY GetPQTY( long lPortNum );
long APIENTRY GetMediaCounter( long lPortNum );
long APIENTRY GetMediaCounterH( long lPortNum );
long APIENTRY GetInitialMediaCount( long lPortNum );
long APIENTRY GetMediaColorOffset( long lPortNum );
long APIENTRY GetMediaLotNo( long lPortNum, LPSTR p );
long APIENTRY GetRfidMediaClass( long lPortNum, LPSTR p );
long APIENTRY GetRfidReserveData( long lPortNum, LPSTR p, DWORD dwPage );
long APIENTRY GetSerialNo( long lPortNum, LPSTR p );
BOOL APIENTRY SetUSBSerialEnable( long lPortNum, DWORD dwEnable );
long APIENTRY GetUSBSerialEnable( long lPortNum );
BOOL APIENTRY SetFirmwUpdateMode( long lPortNum );
BOOL APIENTRY SetFirmwDataWrite( long lPortNum, LPSTR dBuff, DWORD bLen );
BOOL APIENTRY SetColorDataClear( long lPortNum );
BOOL APIENTRY SetColorDataWrite( long lPortNum, LPSTR dBuff, DWORD bLen );
BOOL APIENTRY SetColorDataVersion( long lPortNum, LPSTR dBuff, DWORD bLen );
long APIENTRY GetColorDataVersion( long lPortNum, LPSTR p );
long APIENTRY GetColorDataVersionRes( long lPortNum, LPSTR p, DWORD lType );
long APIENTRY GetColorDataVersionResEX( long lPortNum, LPSTR p, DWORD dwType, DWORD dwMedia );
long APIENTRY GetColorDataChecksum( long lPortNum, LPSTR p );
long APIENTRY GetColorDataChecksumRes( long lPortNum, LPSTR p, DWORD lType  );
long APIENTRY GetColorDataChecksumResEX( long lPortNum, LPSTR p, DWORD dwType, DWORD dwMedia );
BOOL APIENTRY SetCutterMode( long lPortNum, DWORD dwMode );
BOOL APIENTRY SetFullCutterSetup( long lPortNum, long size1, long size2, long size3, long size4, long size5, long size6 );
BOOL APIENTRY SetFullCutterSetupEX( long lPortNum, long size1, long size2, long size3, long size4, long size5, long size6, long dustSize );
BOOL APIENTRY SetStandbyTime( long lPortNum, long lTime );
long APIENTRY GetStandbyTime( long lPortNum );
BOOL APIENTRY SetSetEndKeepMode( long lPortNum, long lMode );
long APIENTRY GetSetEndKeepMode( long lPortNum );
long APIENTRY GetPanoramaPrintable( long lPortNum );
BOOL APIENTRY SetOvercoatFinish ( long lPortNum, DWORD  ovcoat );
BOOL APIENTRY SetCommand( long lPortNum, LPSTR lpCmd, DWORD dwCmdLen );
long APIENTRY GetCommandEX( long lPortNum, LPSTR lpCmd, DWORD dwCmdLen, LPSTR lpRetBuff, DWORD dwRetBuffSize );

BOOL APIENTRY SetPrintSpeed( long lPortNum, long lPrintSpeed );

BOOL APIENTRY SetGammaTable( long lPortNum, LPBYTE lpGammaTable, long lDataLen );
long APIENTRY GetGammaTableChecksum( long lPortNum, LPSTR p );
BOOL APIENTRY SetDataTableClear( long lPortNum );

BOOL APIENTRY SetContPanorama( long lPortNum, DWORD dwContinuous, DWORD dwOverlap );

#ifdef __cplusplus
}
#endif

#endif //_DS820STAT_H_

