Option Strict Off
Option Explicit On
Imports System.Runtime.InteropServices

Module Module1

	Public Const MediaNumMax As Integer = 64	'Number of paper size

	Public Const COLOR_RED As Short = &HFFS
	Public Const COLOR_WHITE As Integer = &HFFFFFF
	Public Const COLOR_BLUE As Integer = &HFFFFC0
	Public Const COLOR_GRAY As Integer = &HC0C0C0

	Const FIXED_LENGTH As Int32 = 32

	Structure PRINTER_DEFAULTS
		Dim pDatatype As Integer
		Dim pDevMode As Integer
		Dim DesiredAccess As Integer
	End Structure

	<DllImport("winspool.Drv", EntryPoint:="OpenPrinterW", _
	   SetLastError:=True, CharSet:=CharSet.Unicode, _
	   ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Function OpenPrinter(ByVal src As String, ByRef hPrinter As IntPtr, ByVal pd As IntPtr) As Boolean
	End Function

	<DllImport("winspool.Drv", EntryPoint:="EnumJobsA", _
	   SetLastError:=True, CharSet:=CharSet.Unicode, _
	   ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Function EnumJobs(ByVal hPrinter As Integer, ByVal FirstJob As Integer, ByVal NoJobs As Integer, _
		ByVal Level As Integer, ByRef pJob As IntPtr, ByVal cdBuf As Integer, ByRef pcbNeeded As Integer, ByRef pcReturned As Integer) As Integer
	End Function

	<DllImport("winspool.Drv", EntryPoint:="ClosePrinter", _
	   SetLastError:=True, CharSet:=CharSet.Unicode, _
	   ExactSpelling:=True, CallingConvention:=CallingConvention.StdCall)> _
	Public Function ClosePrinter(ByVal hPrinter As IntPtr) As Boolean
	End Function

	'---------------------------------------------------------
	' Function - Get printer status
	'---------------------------------------------------------
	Function GetStateStr(ByVal UsbPort As Integer) As String
		Dim state As Integer
		Dim stateStr As String

		stateStr = ""

		state = GetStatus(UsbPort)
		If state And GROUP_USUALLY Then
			Select Case state
				Case STATUS_USUALLY_IDLE : stateStr = "Idle"
				Case STATUS_USUALLY_PRINTING : stateStr = "Printing"
				Case STATUS_USUALLY_STANDSTILL : stateStr = "STANDSTILL"
				Case STATUS_USUALLY_PAPER_END : stateStr = "Paper End"
				Case STATUS_USUALLY_RIBBON_END : stateStr = "Ribbon End"
				Case STATUS_USUALLY_COOLING : stateStr = "Head Cooling Down"
				Case STATUS_USUALLY_MOTCOOLING : stateStr = "Motor Cooling Down"
			End Select
		ElseIf state And GROUP_SETTING Then
			Select Case state
				Case STATUS_SETTING_COVER_OPEN : stateStr = "Cover Open"
				Case STATUS_SETTING_PAPER_JAM : stateStr = "Paper Jam"
				Case STATUS_SETTING_RIBBON_ERR : stateStr = "Ribbon Error"
				Case STATUS_SETTING_PAPER_ERR : stateStr = "Paper definition Error"
				Case STATUS_SETTING_DATA_ERR : stateStr = "Data Error"
				Case STATUS_SETTING_SCRAPBOX_ERR : stateStr = "Scrap Box Error"
			End Select
		ElseIf state And GROUP_HARDWARE Then
			Select Case state
				Case STATUS_HARDWARE_ERR01 : stateStr = "Head Voltage Error"
				Case STATUS_HARDWARE_ERR02 : stateStr = "Head Position Error"
				Case STATUS_HARDWARE_ERR03 : stateStr = "Fan Stop Error"
				Case STATUS_HARDWARE_ERR04 : stateStr = "Cutter Error"
				Case STATUS_HARDWARE_ERR05 : stateStr = "Pinch Roller Error"
				Case STATUS_HARDWARE_ERR06 : stateStr = "Illegal Head Temperature"
				Case STATUS_HARDWARE_ERR07 : stateStr = "Illegal Media Temperature"
				Case STATUS_HARDWARE_ERR08 : stateStr = "Ribbon Tension Error"
				Case STATUS_HARDWARE_ERR09 : stateStr = "RFID Module Error"
				Case STATUS_HARDWARE_ERR10 : stateStr = "Illegal Motor Temperature"
			End Select
		ElseIf state And GROUP_SYSTEM Then
			stateStr = "SYSTEM ERROR"
		ElseIf state And GROUP_FLSHPROG Then
			stateStr = "FLSHPROG MODE"
		ElseIf state < 0 Then
			stateStr = " ---"
		End If

		GetStateStr = stateStr

	End Function

	'----------------------------------------------------------------------------------
	' Function - File Access
	'----------------------------------------------------------------------------------

	Function getFilePath(ByVal strOrg As String, ByVal defaultDir As String, ByRef strFilePath As String) As Boolean
		'param check
		If strOrg Is Nothing Then
			getFilePath = False
			Exit Function
		End If

		If strOrg.Length <= 0 Then
			getFilePath = False
			Exit Function
		End If

		If strFilePath Is Nothing Then
			getFilePath = False
			Exit Function
		End If

		strFilePath = ""

		'check original string
		If strOrg.IndexOf(":") <> -1 Then
			'full path file name
			strFilePath = strOrg
		Else
			'other
			If defaultDir.Length = 0 Then
				strFilePath = My.Application.Info.DirectoryPath & "\" & strOrg
			Else
				strFilePath = My.Application.Info.DirectoryPath & "\" & defaultDir & "\" & strOrg
			End If
		End If

		'check file exist
		getFilePath = System.IO.File.Exists(strFilePath)

	End Function

	Function readCSVFile(ByVal strFilePath As String, ByRef NumOfLine As Int32, ByRef csvData As Object) As Boolean
		Dim bRet As Boolean = False
		Dim arrCsvData As New System.Collections.ArrayList()
		Dim swText As FileIO.TextFieldParser = Nothing
		Dim fields() As String

		Try
			swText = New FileIO.TextFieldParser(strFilePath, System.Text.Encoding.Default)

			swText.TextFieldType = FileIO.FieldType.Delimited

			swText.Delimiters = New String() {","}

			swText.CommentTokens = New String() {"#"}

			swText.HasFieldsEnclosedInQuotes = True

			swText.TrimWhiteSpace = True

			While Not swText.EndOfData
				fields = swText.ReadFields()
				arrCsvData.Add(fields)
			End While

		Catch ex As Exception
			'error
			MsgBox(ex.Message, MsgBoxStyle.OkOnly, "readCSVFile()")
		Finally
			If swText IsNot Nothing Then
				'release
				swText.Close()
				swText.Dispose()
			End If
		End Try

		' copy data
		NumOfLine = arrCsvData.Count()
		If NumOfLine > 0 Then
			bRet = True
			csvData = arrCsvData.Clone()
		End If

		Return bRet

	End Function

	'----------------------------------------------------------------------------------
	' Function - Data Convert
	'----------------------------------------------------------------------------------

	Sub assignData(ByRef byteArray() As Byte, ByRef offset As Integer, ByVal srcData() As Byte, ByVal isBigEndian As Boolean)

		If isBigEndian Then
			byteArray(offset) = srcData(1)
			offset += 1
			byteArray(offset) = srcData(0)
			offset += 1
		Else
			byteArray(offset) = srcData(0)
			offset += 1
			byteArray(offset) = srcData(1)
			offset += 1
		End If

	End Sub

	Function Convert_byteToUint16(ByRef srcData() As Byte, ByVal pos As Integer, ByVal size As Integer, ByVal isBigEndian As Boolean) As UInt16

		Dim dstData(size - 1) As Byte

		'Copy to work buffer.
		System.Array.Copy(srcData, pos, dstData, 0, size)

		If isBigEndian Then
			System.Array.Reverse(dstData)
		End If

		'Convert data type to uint16.
		Convert_byteToUint16 = BitConverter.ToUInt16(dstData, 0)

	End Function

	'----------------------------------------------------------------------------------
	' Function - Printer Command
	'----------------------------------------------------------------------------------

	Function SetCommandStr(ByVal cmdStr As String, ByVal cmdLen As Int32) As String
		Dim tempStr As String

		' add <ESC>P
		tempStr = String.Format("{0}P{1}", Chr(&H1B), cmdStr)
		' add blank(<SPACE>:0x20)
		If tempStr.Length < FIXED_LENGTH Then
			Dim i As Int32
			Dim fillNum As Int32
			fillNum = FIXED_LENGTH - tempStr.Length
			For i = 1 To fillNum
				tempStr = tempStr & Chr(&H20)
			Next
		End If
		' add ...<CR><NULL>
		If tempStr.Length < cmdLen Then
			tempStr = tempStr & vbCr
			If tempStr.Length < cmdLen Then
				Dim i As Int32
				Dim fillNum As Int32
				fillNum = cmdLen - tempStr.Length
				For i = 1 To fillNum
					tempStr = tempStr & vbNullChar
				Next
			End If
		End If

		SetCommandStr = tempStr
	End Function

End Module
